# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Geometry/PolarArc.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Geometry/PolarArc
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Surface import Surface

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Geometry.PolarArc.get_lines import get_lines
except ImportError as error:
    get_lines = error

try:
    from ..Methods.Geometry.PolarArc.rotate import rotate
except ImportError as error:
    rotate = error

try:
    from ..Methods.Geometry.PolarArc.translate import translate
except ImportError as error:
    translate = error

try:
    from ..Methods.Geometry.PolarArc.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Geometry.PolarArc.comp_length import comp_length
except ImportError as error:
    comp_length = error

try:
    from ..Methods.Geometry.PolarArc.discretize import discretize
except ImportError as error:
    discretize = error

try:
    from ..Methods.Geometry.PolarArc.get_patches import get_patches
except ImportError as error:
    get_patches = error

try:
    from ..Methods.Geometry.PolarArc.comp_surface import comp_surface
except ImportError as error:
    comp_surface = error

try:
    from ..Methods.Geometry.PolarArc.comp_point_ref import comp_point_ref
except ImportError as error:
    comp_point_ref = error


from ._check import InitUnKnowClassError


class PolarArc(Surface):
    """PolarArc defined by  the center of object(point_ref), the label, the angle and the height"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Geometry.PolarArc.get_lines
    if isinstance(get_lines, ImportError):
        get_lines = property(
            fget=lambda x: raise_(
                ImportError("Can't use PolarArc method get_lines: " + str(get_lines))
            )
        )
    else:
        get_lines = get_lines
    # cf Methods.Geometry.PolarArc.rotate
    if isinstance(rotate, ImportError):
        rotate = property(
            fget=lambda x: raise_(
                ImportError("Can't use PolarArc method rotate: " + str(rotate))
            )
        )
    else:
        rotate = rotate
    # cf Methods.Geometry.PolarArc.translate
    if isinstance(translate, ImportError):
        translate = property(
            fget=lambda x: raise_(
                ImportError("Can't use PolarArc method translate: " + str(translate))
            )
        )
    else:
        translate = translate
    # cf Methods.Geometry.PolarArc.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use PolarArc method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Geometry.PolarArc.comp_length
    if isinstance(comp_length, ImportError):
        comp_length = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use PolarArc method comp_length: " + str(comp_length)
                )
            )
        )
    else:
        comp_length = comp_length
    # cf Methods.Geometry.PolarArc.discretize
    if isinstance(discretize, ImportError):
        discretize = property(
            fget=lambda x: raise_(
                ImportError("Can't use PolarArc method discretize: " + str(discretize))
            )
        )
    else:
        discretize = discretize
    # cf Methods.Geometry.PolarArc.get_patches
    if isinstance(get_patches, ImportError):
        get_patches = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use PolarArc method get_patches: " + str(get_patches)
                )
            )
        )
    else:
        get_patches = get_patches
    # cf Methods.Geometry.PolarArc.comp_surface
    if isinstance(comp_surface, ImportError):
        comp_surface = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use PolarArc method comp_surface: " + str(comp_surface)
                )
            )
        )
    else:
        comp_surface = comp_surface
    # cf Methods.Geometry.PolarArc.comp_point_ref
    if isinstance(comp_point_ref, ImportError):
        comp_point_ref = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use PolarArc method comp_point_ref: " + str(comp_point_ref)
                )
            )
        )
    else:
        comp_point_ref = comp_point_ref
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, angle=1, height=1, point_ref=0, label="", init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "angle" in list(init_dict.keys()):
                angle = init_dict["angle"]
            if "height" in list(init_dict.keys()):
                height = init_dict["height"]
            if "point_ref" in list(init_dict.keys()):
                point_ref = init_dict["point_ref"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
        # Set the properties (value check and convertion are done in setter)
        self.angle = angle
        self.height = height
        # Call Surface init
        super(PolarArc, self).__init__(point_ref=point_ref, label=label)
        # The class is frozen (in Surface init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        PolarArc_str = ""
        # Get the properties inherited from Surface
        PolarArc_str += super(PolarArc, self).__str__()
        PolarArc_str += "angle = " + str(self.angle) + linesep
        PolarArc_str += "height = " + str(self.height) + linesep
        return PolarArc_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Surface
        if not super(PolarArc, self).__eq__(other):
            return False
        if other.angle != self.angle:
            return False
        if other.height != self.height:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Surface
        diff_list.extend(super(PolarArc, self).compare(other, name=name))
        if other._angle != self._angle:
            diff_list.append(name + ".angle")
        if other._height != self._height:
            diff_list.append(name + ".height")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Surface
        S += super(PolarArc, self).__sizeof__()
        S += getsizeof(self.angle)
        S += getsizeof(self.height)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Surface
        PolarArc_dict = super(PolarArc, self).as_dict(**kwargs)
        PolarArc_dict["angle"] = self.angle
        PolarArc_dict["height"] = self.height
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        PolarArc_dict["__class__"] = "PolarArc"
        return PolarArc_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.angle = None
        self.height = None
        # Set to None the properties inherited from Surface
        super(PolarArc, self)._set_None()

    def _get_angle(self):
        """getter of angle"""
        return self._angle

    def _set_angle(self, value):
        """setter of angle"""
        check_var("angle", value, "float", Vmin=0)
        self._angle = value

    angle = property(
        fget=_get_angle,
        fset=_set_angle,
        doc=u"""Polar angle

        :Type: float
        :min: 0
        """,
    )

    def _get_height(self):
        """getter of height"""
        return self._height

    def _set_height(self, value):
        """setter of height"""
        check_var("height", value, "float", Vmin=0)
        self._height = value

    height = property(
        fget=_get_height,
        fset=_set_height,
        doc=u"""The Heigth of the PolarAngle

        :Type: float
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-
from logging import WARNING
import traceback
import sys

if sys.version_info > (3, 0):
    from io import StringIO
else:
    from StringIO import StringIO

from os import stat, remove
from datetime import datetime
from numpy import nan


def evaluate(solver, indiv):
    """Evaluate the individual according to the solver method

    Parameters
    ----------
    solver : Solver
        optimization solver
    indiv : individual
        individual to evaluate

    Returns
    -------
    evaluation_failure : bool
        failure of the evaluation
    """
    # Get solver logger
    logger = solver.get_logger()

    tb = StringIO()  # to store the traceback in case of error

    logger.debug("Design variables :")
    for i, design_variable in enumerate(indiv.design_var_name_list):
        logger.debug(design_variable + " : " + str(indiv[i]))

    try:
        # Run the preprocessing
        if solver.problem.preprocessing is not None:
            solver.problem.preprocessing(indiv.output.simu)

        if solver.problem.eval_func == None:
            indiv.output.simu.run()
        else:
            solver.problem.eval_func(indiv.output)

        # Add the fitness values, handle exception for each fitness
        fitness = []
        for obj_func in solver.problem.obj_func:
            try:
                fitness.append(float(obj_func.keeper(indiv.output)))
            # Raise KeyboardInterrupt to stop optimization
            except KeyboardInterrupt:
                raise KeyboardInterrupt

            # Except error and try to compute the error_keeper
            except Exception as err:
                logger.warning(
                    "Objectiv computation " + obj_func.name + " failed:" + str(err)
                )
                if obj_func.error_keeper is None:  # Set fitness value as infinity
                    fitness.append(float("inf"))
                else:
                    try:
                        fitness.append(float(obj_func.error_keeper(indiv.output)))
                    # Raise KeyboardInterrupt to stop optimization
                    except KeyboardInterrupt:
                        raise KeyboardInterrupt
                    # Set the fitness value as infinity
                    except Exception as err:
                        logger.warning(
                            "Objectiv error computation "
                            + obj_func.name
                            + " failed:"
                            + str(err)
                        )
                        fitness.append(float("inf"))

        # Add fitness values to indiv object
        indiv.fitness.values = fitness

        # Execute the DataKeepers
        for datakeeper in solver.problem.datakeeper_list:
            try:
                datakeeper.result.append(datakeeper.keeper(indiv.output))
            except KeyboardInterrupt:
                raise KeyboardInterrupt("Stopped by the user.")
            except Exception as err:
                logger.warning(
                    "DataKeeper"
                    + datakeeper.name
                    + ".keeper execution failed:"
                    + str(err)
                )
                if datakeeper.error_keeper:
                    try:
                        datakeeper.result.append(
                            datakeeper.error_keeper(indiv.output.simu)
                        )
                    except KeyboardInterrupt:
                        raise KeyboardInterrupt("Stopped by the user.")
                    except Exception as err:
                        logger.warning(
                            "DataKeeper"
                            + datakeeper.name
                            + ".error_keeper execution failed:"
                            + str(err)
                        )
                        datakeeper.result.append(nan)
                else:
                    datakeeper.result.append(nan)

        indiv.is_simu_valid = True

        evaluation_failure = False  # Evaluation succeed

    except KeyboardInterrupt:
        raise KeyboardInterrupt("Stopped by the user.")

    except:
        # Logging
        print("The following simulation failed :", file=tb)

        if logger.level > 10:  # Log design variables values if it is not already done
            print("Design variables :", file=tb)
            for i, design_variable in enumerate(indiv.design_var_name_list):
                print(design_variable + " : " + str(indiv[i]), file=tb)

        # Log the simulation error
        traceback.print_exc(file=tb)
        logger.warning(tb.getvalue())

        # Set fitness as inf
        indiv.fitness.values = [float("inf") for _ in solver.problem.obj_func]
        indiv.is_simu_valid = False

        # Execute the DataKeepers
        for datakeeper in solver.problem.datakeeper_list:
            if datakeeper.error_keeper:
                try:
                    datakeeper.result.append(datakeeper.error_keeper(indiv.output.simu))
                except KeyboardInterrupt:
                    raise KeyboardInterrupt("Stopped by the user.")
                except Exception as err:
                    logger.warning(
                        "DataKeeper"
                        + datakeeper.name
                        + ".error_keeper execution failed:"
                        + str(err)
                    )
                    datakeeper.result.append(nan)
            else:
                datakeeper.result.append(nan)

        # Reset standard output and error
        evaluation_failure = True  # Evaluation failed

    return evaluation_failure



# -*- coding: utf-8 -*-

from ....Methods.Simulation.Input import InputError


def run(self):
    """Run the Structural module"""
    if self.parent is None:
        raise InputError(
            "ERROR: The Structural object must be in a Simulation object to run"
        )
    if self.parent.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )

    output = self.parent.parent

    self.comp_axes(output)

    group = self.force.group  # Magnetic force target

    # Init model, generate or import mechanical mesh
    self.init_mechanical_model(output, group)

    # Compute the magnetic force according to the Force model
    self.force.comp_force(output)



# -*- coding: utf-8 -*-

# File generated according to PCondType21.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.FloatEdit import FloatEdit
from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect
from ......GUI.Dialog.DMachineSetup.SBar.WBarOut.WBarOut import WBarOut


class Ui_PCondType21(object):
    def setupUi(self, PCondType21):
        if not PCondType21.objectName():
            PCondType21.setObjectName(u"PCondType21")
        PCondType21.resize(416, 124)
        self.horizontalLayout = QHBoxLayout(PCondType21)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.lf_Hbar = FloatEdit(PCondType21)
        self.lf_Hbar.setObjectName(u"lf_Hbar")
        self.lf_Hbar.setMinimumSize(QSize(70, 0))
        self.lf_Hbar.setMaximumSize(QSize(100, 20))

        self.gridLayout.addWidget(self.lf_Hbar, 0, 1, 1, 1)

        self.in_Hbar = QLabel(PCondType21)
        self.in_Hbar.setObjectName(u"in_Hbar")
        self.in_Hbar.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Hbar, 0, 0, 1, 1)

        self.unit_Hbar = QLabel(PCondType21)
        self.unit_Hbar.setObjectName(u"unit_Hbar")
        self.unit_Hbar.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Hbar, 0, 2, 1, 1)

        self.lf_Wbar = FloatEdit(PCondType21)
        self.lf_Wbar.setObjectName(u"lf_Wbar")
        self.lf_Wbar.setMinimumSize(QSize(70, 0))
        self.lf_Wbar.setMaximumSize(QSize(100, 20))

        self.gridLayout.addWidget(self.lf_Wbar, 1, 1, 1, 1)

        self.unit_Wbar = QLabel(PCondType21)
        self.unit_Wbar.setObjectName(u"unit_Wbar")
        self.unit_Wbar.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.unit_Wbar, 1, 2, 1, 1)

        self.in_Wbar = QLabel(PCondType21)
        self.in_Wbar.setObjectName(u"in_Wbar")
        self.in_Wbar.setMinimumSize(QSize(40, 0))

        self.gridLayout.addWidget(self.in_Wbar, 1, 0, 1, 1)

        self.verticalLayout_2.addLayout(self.gridLayout)

        self.w_mat = WMatSelect(PCondType21)
        self.w_mat.setObjectName(u"w_mat")

        self.verticalLayout_2.addWidget(self.w_mat)

        self.horizontalLayout.addLayout(self.verticalLayout_2)

        self.w_out = WBarOut(PCondType21)
        self.w_out.setObjectName(u"w_out")

        self.horizontalLayout.addWidget(self.w_out)

        QWidget.setTabOrder(self.lf_Hbar, self.lf_Wbar)

        self.retranslateUi(PCondType21)

        QMetaObject.connectSlotsByName(PCondType21)

    # setupUi

    def retranslateUi(self, PCondType21):
        PCondType21.setWindowTitle(
            QCoreApplication.translate("PCondType21", u"Form", None)
        )
        self.in_Hbar.setText(QCoreApplication.translate("PCondType21", u"Hbar :", None))
        self.unit_Hbar.setText(QCoreApplication.translate("PCondType21", u"m", None))
        self.unit_Wbar.setText(QCoreApplication.translate("PCondType21", u"m", None))
        self.in_Wbar.setText(QCoreApplication.translate("PCondType21", u"Wbar :", None))

    # retranslateUi



# -*- coding: utf-8 -*-
import numpy as np


def get_field(self, *args):
    """Get the value of variables stored in Solution.

    Parameters
    ----------
    self : Solution
        an Solution object
    *args: list of strings
        List of axes requested by the user, their units and values (optional)

    Returns
    -------
    field: ndarray
        an ndarray of field values

    """
    if args is None:
        args = dict()
    else:
        if len(args) == 1 and type(args[0]) == tuple:
            args = args[0]

    field = self.field
    if self.axis_name is not None:
        nb_axis = len(self.axis_name)
        for i in range(nb_axis):
            ax = np.where(np.array(field.shape) == self.axis_size[i])[0][0]
            ax_name = self.axis_name[i]

            for axis_str in args:
                if ax_name in axis_str:
                    if "[" in axis_str:
                        elems = axis_str.split("[")
                        ind_str = elems[1].strip("]")

                        # Range of indices
                        if ":" in ind_str:
                            elems2 = ind_str.split(":")
                            indices = [
                                i for i in range(int(elems2[0]), int(elems2[1]) + 1)
                            ]
                        # List of indices
                        elif "," in ind_str:
                            indices = [int(x) for x in ind_str.split(",")]
                        # Single index
                        else:
                            indices = [int(ind_str)]

                        field = np.take(field, indices=indices, axis=ax)

        return field



# -*- coding: utf-8 -*-
"""File generated according to PWSlot25/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWSlot.PWSlot25.Ui_PWSlot25 import Ui_PWSlot25


class Gen_PWSlot25(Ui_PWSlot25):
    def setupUi(self, PWSlot25):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PWSlot25.setupUi(self, PWSlot25)
        # Setup of in_W3
        txt = self.tr(u"""Teeth bottom width""")
        self.in_W3.setWhatsThis(txt)
        self.in_W3.setToolTip(txt)

        # Setup of lf_W3
        self.lf_W3.validator().setBottom(0)
        txt = self.tr(u"""Teeth bottom width""")
        self.lf_W3.setWhatsThis(txt)
        self.lf_W3.setToolTip(txt)

        # Setup of in_W4
        txt = self.tr(u"""Teeth top width""")
        self.in_W4.setWhatsThis(txt)
        self.in_W4.setToolTip(txt)

        # Setup of lf_W4
        self.lf_W4.validator().setBottom(0)
        txt = self.tr(u"""Teeth top width""")
        self.lf_W4.setWhatsThis(txt)
        self.lf_W4.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Slot top height""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Slot top height""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)

        # Setup of in_H2
        txt = self.tr(u"""Slot bottom height""")
        self.in_H2.setWhatsThis(txt)
        self.in_H2.setToolTip(txt)

        # Setup of lf_H2
        self.lf_H2.validator().setBottom(0)
        txt = self.tr(u"""Slot bottom height""")
        self.lf_H2.setWhatsThis(txt)
        self.lf_H2.setToolTip(txt)



# -*- coding: utf-8 -*-
from ...definitions import config_dict

PHASE_COLORS = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"]


def find_wind_phase_color(label, wind_mat):
    """Returns Color phase of the Winding surface

    Parameters
    ----------
    label : str
        the label of the surface

    wind_mat : numpy.ndarray
        A matrix [Nrad,Ntan,Zs,qs] representing the winding

    Returns
    -------
    color: str
        Color of the zone

    """
    st = label.split("_")
    Nrad = int(st[2][1:])
    Ntan = int(st[3][1:])
    Zs = int(st[4][1:])
    if wind_mat is not None:
        q = get_phase_id(wind_mat, Nrad, Ntan, Zs)
        if q is None:  # No phase => White
            color = "w"
        else:
            color = PHASE_COLORS[q]
    else:
        color = PHASE_COLORS[0]

    return color


def get_phase_id(wind_mat, Nrad, Ntan, Zs):
    """Return the id of the corresponding phase for the zone (Nrad,Ntan,Zs)

    Parameters
    ----------
    wind_mat : numpy.ndarray
        A matrix [Nrad,Ntan,Zs,qs] representing the winding
    Nrad : int
        Zone radial coordinate
    Ntan : int
        Zone tagential coordinate
    Zs : int
        Zone slot number coordinate

    Returns
    -------
    q_id: int
        Id of the phase

    """
    A = wind_mat[Nrad, Ntan, Zs, :]
    for zz in range(len(A)):
        if A[zz] != 0:
            return zz
    return None  # If all the phase are at 0 : the zone is empty



# -*- coding: utf-8 -*-

from numpy import arcsin, pi


def comp_alpha(self):
    """The opening angle with a W3 teeth width and Rext - H0 radius

    Parameters
    ----------
    self : HoleM52
        A HoleM52 object

    Returns
    -------
    alpha: float
        Angle between P1 and P9 (cf schematics) [rad]

    """
    Rext = self.get_Rext()

    alpha_tooth = 2 * arcsin(self.W3 / (2 * (Rext - self.H0)))
    slot_pitch = 2 * pi / self.Zh

    return slot_pitch - alpha_tooth



from PySide2.QtCore import QSize
from PySide2.QtWidgets import QGroupBox, QLabel, QVBoxLayout, QWidget

from ......GUI import gui_option


class WCondOut(QGroupBox):
    """Setup of QGroupBox for output for Winding Conductor"""

    def __init__(self, parent=None):
        """Initialize the widget"""

        QGroupBox.__init__(self, parent)
        # Set main widget
        self.u = gui_option.unit
        self.setTitle(self.tr("Output"))
        self.setMinimumSize(QSize(200, 0))
        self.setObjectName("g_output")
        self.layout = QVBoxLayout(self)
        self.layout.setObjectName("layout")
        # The widget is composed of several QLabel in a vertical layout
        self.out_H = QLabel(self)
        self.out_H.setObjectName("out_H")
        self.layout.addWidget(self.out_H)

        self.out_W = QLabel(self)
        self.out_W.setObjectName("out_W")
        self.layout.addWidget(self.out_W)

        self.out_S = QLabel(self)
        self.out_S.setObjectName("out_S")
        self.layout.addWidget(self.out_S)

        self.out_Sact = QLabel(self)
        self.out_Sact.setObjectName("out_Sact")
        self.layout.addWidget(self.out_Sact)

        self.out_K = QLabel(self)
        self.out_K.setObjectName("out_K")
        self.layout.addWidget(self.out_K)

    def comp_output(self):
        """Update the Output with the computed values

        Parameters
        ----------
        self : WCondOut
            A WCondOut object
        """

        lam = self.parent().parent().lam
        H_txt = self.tr("Hcond = ")
        W_txt = self.tr("Wcond = ")
        S_txt = self.tr("Scond = ")
        Sa_txt = self.tr("Scond_active = ")
        if lam.is_stator:
            K_txt = self.tr("Ksfill = ")
        else:
            K_txt = self.tr("Krfill = ")
        # We compute the output only if the slot is correctly set
        if self.parent().parent().check() is None:
            # Compute all the needed output as string
            H = format(self.u.get_m(lam.winding.conductor.comp_height()), ".4g")
            W = format(self.u.get_m(lam.winding.conductor.comp_width()), ".4g")
            S = format(self.u.get_m2(lam.winding.conductor.comp_surface()), ".4g")
            Sact = format(
                self.u.get_m2(lam.winding.conductor.comp_surface_active()), ".4g"
            )
            try:
                K = "%.2f" % (lam.comp_fill_factor() * 100)
            except Exception:  # Unable to compute the fill factor (Not set)
                K = "?"

            # Update the GUI to display the Output
            self.out_H.setText(H_txt + H + " [" + self.u.get_m_name() + "]")
            self.out_W.setText(W_txt + W + " [" + self.u.get_m_name() + "]")
            self.out_S.setText(S_txt + S + " [" + self.u.get_m2_name() + "]")
            self.out_Sact.setText(Sa_txt + Sact + " [" + self.u.get_m2_name() + "]")
            self.out_K.setText(K_txt + K + " %")
        else:
            # We can't compute the output => We erase the previous version
            # (that way the user know that something is wrong)
            self.out_H.setText(H_txt + "?")
            self.out_W.setText(W_txt + "?")
            self.out_S.setText(S_txt + "?")
            self.out_Sact.setText(Sa_txt + "?")
            self.out_K.setText(K_txt + "? %")



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/FluxLink.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/FluxLink
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

from ._check import InitUnKnowClassError


class FluxLink(FrozenClass):
    """Electric module: Flux Linkage"""

    VERSION = 1

    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in two ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for Matrix, None will initialise the property with an empty Matrix
            for pyleecan type, None will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary wiht every properties as keys

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_dict is not None:  # Initialisation by dict
            assert init_dict == {"__class__": "FluxLink"}
        if init_str is not None:  # Initialisation by str
            assert type(init_str) is str
        # The class is frozen, for now it's impossible to add new properties
        self.parent = None
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        FluxLink_str = ""
        if self.parent is None:
            FluxLink_str += "parent = None " + linesep
        else:
            FluxLink_str += "parent = " + str(type(self.parent)) + " object" + linesep
        return FluxLink_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        FluxLink_dict = dict()
        # The class name is added to the dict for deserialisation purpose
        FluxLink_dict["__class__"] = "FluxLink"
        return FluxLink_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""
        pass



# -*- coding: utf-8 -*-

from numpy import cos, sin


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW15
        A SlotW15 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    Rbo = self.get_Rbo()

    Swind = self.comp_surface_active()

    # The bottom is an arc
    alpha = self.comp_angle_opening()
    Sarc = (Rbo ** 2.0) / 2.0 * (alpha - sin(alpha))
    Harc = float(Rbo * (1 - cos(alpha / 2)))
    S1 = (self.H0 + Harc) * self.W0

    # Because Slamination = S - Zs * Sslot
    if self.is_outwards():
        return S1 + Swind - Sarc



# -*- coding: utf-8 -*-


def comp_surface(self):
    """Compute the surface of the conductor

    Parameters
    ----------
    self : CondType22
        A CondType22 object

    Returns
    -------
    S: float
        Surface of the conductor (with insulation) [m**2]

    """

    return self.Sbar



# -*- coding: utf-8 -*-
from ....Classes.LossModel import LossModel
from ....Methods.Simulation.Loss import LossError


def remove_model(self, part_label, index):
    """Remove a loss model from the list of loss models.

    Parameter
    ---------
    self : Loss
        Loss object

    part_label : str
        Part label of the model to remove.

    index : int
        Index of the model, e.g. if there are multiple 'stator core' loss models

    Return
    ------

    is_success : bool
        Could the model be removed?

    """
    logger = self.get_logger()

    # check if dict key exists
    if part_label not in self.model_index.keys():
        logger.warning(
            f"Loss.remove_model(): Loss model for part '{part_label}' "
            + "doesn't exists. No model removed."
        )
        return False

    # check if sub dict key exists
    if index not in self.model_index[part_label].keys():
        logger.warning(
            f"Loss.remove_model(): Loss model for part '{part_label}' "
            + f"with index {index} doesn't exists. No model removed."
        )
        return False

    # for simplicity set model to None
    else:
        ii = self.model_index[part_label][index]
        self.model_list[ii] = None
        self.model_index[part_label].pop(index)

        return True



# -*- coding: utf-8 -*-

from PySide2 import QtCore, QtGui, QtWidgets

# === TreeView ========================================================================
class TreeView(QtWidgets.QTreeView):
    def __init__(self):
        super(TreeView, self).__init__()

        # self.__typeWidgets = {}
        # self.__typeWrappers = {}

        model = QtGui.QStandardItemModel(0, 1)
        model.setHorizontalHeaderLabels(
            ["Name", "Value", "Unit", "Class", "Description"]
        )

        self.rootNode = model.invisibleRootItem()

        self.setModel(model)
        self.setColumnWidth(0, 150)

        self.setAlternatingRowColors(True)

    def generate(self, data, parent=None):
        if parent is None:
            parent = self.rootNode
            self.rootNode.removeRows(0, self.rootNode.rowCount())
        if hasattr(data, "as_dict"):
            self.generate_branch(data, parent)
        else:
            pass

    def generate_branch(self, data, parent):
        for attr in data.as_dict():
            if attr[0] != "_":
                _attr = getattr(data, attr)
                # pyleecan object attributes
                if hasattr(_attr, "as_dict"):
                    branch = QtGui.QStandardItem(attr)
                    branch.setEditable(False)
                    value = QtGui.QStandardItem("")
                    value.setEditable(False)
                    unit = QtGui.QStandardItem("")
                    cls_name = QtGui.QStandardItem(type(_attr).__name__)
                    # class description rathern than attribure description
                    cls_doc = (
                        getattr(type(data), attr).__doc__
                        if _attr.__doc__ is None
                        else _attr.__doc__
                    )
                    cls_doc = QtGui.QStandardItem(cls_doc)
                    parent.appendRow([branch, value, unit, cls_name, cls_doc])
                    self.generate(_attr, parent=branch)

                # float, int, str attributes
                elif isinstance(_attr, (float, int, str)):
                    attribute = QtGui.QStandardItem(attr)
                    attribute.setEditable(False)
                    value = QtGui.QStandardItem(str(getattr(data, attr)))
                    value.setEditable(False)
                    unit = QtGui.QStandardItem("na")
                    cls_name = QtGui.QStandardItem(type(_attr).__name__)
                    cls_doc = getattr(
                        type(data), attr
                    ).__doc__  # tc.__class__.prop.__doc__
                    cls_doc = QtGui.QStandardItem(cls_doc)
                    parent.appendRow([attribute, value, unit, cls_name, cls_doc])

                # todo: list, dict, ndarray attributes
                else:
                    pass



# -*- coding: utf-8 -*-


def get_angle_offset_initial(self):
    """Return the difference between the d axis angle of the stator and the rotor

    Parameters
    ----------
    self : Output
        an Output object

    Returns
    -------
    angle_offset_initial: float
        difference between the d axis angle of the stator and the rotor [rad]

    """

    # Already available => Return
    if (
        self.geo.angle_offset_initial is not None
        and self.geo.angle_offset_initial.size > 0
    ):
        return self.geo.angle_offset_initial
    else:  # Compute
        self.geo.angle_offset_initial = self.simu.machine.comp_angle_offset_initial()
        return self.geo.angle_offset_initial



def comp_FEMM_Jcus(lam, cname, I, j_t0, is_mmf):
    """Compute the current density for FEMM [A/mm2]"""

    Npcpp = lam.winding.Npcpp  # number of parallel circuits  per phase (maximum 2p)
    (Nrad, Ntan) = lam.winding.get_dim_wind()
    Nwpc = Nrad * Ntan  # total number of wires / strands in parallel per coil
    Ksfill = lam.comp_fill_factor()
    Swire = lam.winding.conductor.comp_surface_active()
    # Decode the label
    q_id = int(cname[2:-1])
    if cname[-1] == "-":
        s = -1
    else:
        s = 1

    if I is None:
        Jcus = 0
    else:
        Jcus = s * 1e-6 * Ksfill * (I[q_id, j_t0] / Npcpp) * is_mmf / Swire / Nwpc

    return Jcus



# -*- coding: utf-8 -*-


def comp_width_opening(self):
    """Compute the average opening width of the Slot

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    W0: float
        Average opening width of the slot [m]
    """

    line_list = self.build_geometry()
    Z1 = line_list[0].get_begin()
    Z2 = line_list[-1].get_end()

    return abs(Z2 - Z1)



# -*- coding: utf-8 -*-


def clean(self, clean_level=1):
    """Clean MagElmer outputs depending on cleaning level

    Parameters
    ----------
    self : OutMagElmer
        the OutMagElmer object to update
    clean_level : int
        Value to indicate which fields to clean in OutMagElmer (default=1/min=0/max=4)

    """

    # if clean_level = 0:
    # keep all internal outputs

    if clean_level > 0:
        # clean FEA_dict
        self.FEA_dict = None



# -*- coding: utf-8 -*-

from os.path import join, split
from numpy import pi

from ....Classes.Section import Section
from ....Classes.SolverInputFile import SolverInputFile
from ....Methods.Elmer.Section import File, Variable, MATC


# constants
# TODO set these constants in dependence of actual number of bodies, i.e. one set of
#     respective sections per body
# TODO set magnet material properties by utilizing label 'translation' dict
#     therefore a 'get_material_by_label' function would be very handy (unite with FEMM)
ID_LAM = 1
ID_MAG = 2


def gen_case(self, output, mesh_names):
    """Setup the Elmer Case file (.sif file)"""

    # get the save path
    save_dir = self.get_path_save_fea(output)

    sif_file = join(save_dir, "case.sif")

    # save solver start info
    start_file = join(save_dir, "ELMERSOLVER_STARTINFO")

    with open(start_file, "wt") as fout:
        fout.write(split(sif_file)[1])

    # generate list of mesh names
    names = []
    for value in mesh_names.values():
        names.extend(value)

    # --- prepare sections ----------------------------------------------------------- #
    # --- bodies --------------------------------------------------------------------- #
    bodies = []
    for name in names:
        if "Body" in name:
            body = Section(section="Body", id=len(bodies) + 1)
            body["name"] = name
            body["Equation"] = 1
            if "Magnet" in name:
                body["Material"] = ID_MAG
            elif "Lamination" in name:
                body["Material"] = ID_LAM
            body["Body Force"] = 1

            bodies.append(body)

    # --- simulation ----------------------------------------------------------------- #
    sim = Section(section="Simulation")
    sim["Max Output Level"] = 5
    sim["Coordinate System"] = "Cartesian 2D"
    sim["Coordinate Mapping"] = [1, 2, 3]
    sim["Simulation Type"] = "Steady State"
    sim["Steady State Max Iterations"] = 1
    sim["Initialize Dirichlet Conditions"] = False
    # sim["Output File"] = File("simulation.result")
    sim["Use Mesh Names"] = True

    # --- constants ------------------------------------------------------------------ #
    const = Section(section="Constants")
    const["Gravity"] = [0.0, -1.0, 0.0, 9.82]

    # --- solvers -------------------------------------------------------------------- #
    solver_list = []

    # solver - move mesh before simulation
    i = 1
    solver = Section(section="Solver", id=i)
    solver.comment = "Moves the magnets as defined in the body force section"
    solver["Exec Solver"] = "Before all"
    solver["Equation"] = "MeshDeform"
    solver["Procedure"] = [File("RigidMeshMapper"), File("RigidMeshMapper")]
    solver_list.append(solver)

    # solver - elasticity simulation
    i += 1
    solver = Section(section="Solver", id=i)
    solver["Equation"] = "Linear Elasticity"
    solver["Procedure"] = [File("ElasticSolve"), File("ElasticSolver")]
    solver["Variable"] = "-dofs 2 Displacement"
    solver["Linear System Solver"] = "Iterative"
    solver["Linear System Preconditioning"] = "ILU3"
    solver["Linear System Residual Output"] = 10
    solver["Linear System Max Iterations"] = 400
    solver["Linear System Iterative Method"] = "GCR"
    solver["Linear System Convergence Tolerance"] = 1.0e-9
    solver["Linear System Abort Not Converged"] = False
    solver["Linear System Residual Mode"] = True
    solver["Nonlinear System Convergence Tolerance"] = 1.0e-7
    solver["Nonlinear System Max Iterations"] = 20
    solver["Nonlinear System Relaxation Factor"] = 1.0
    # solver['Displace Mesh'] = True
    solver["Calculate Principal"] = True
    solver["Calculate Loads"] = True
    solver["Calculate Stresses"] = True
    solver["Calculate Strains"] = True
    solver["Apply Contact BCs"] = True
    solver["Stabilize"] = True
    solver["Elasticity Solver Linear"] = True
    solver["Optimize Bandwidth"] = True
    solver_list.append(solver)

    # solver - save results as vtu if requested
    i += 1
    solver = Section(section="Solver", id=i)
    solver["Exec Solver"] = "After Simulation" if self.is_save_FEA else "Never"
    solver["Equation"] = "result output"
    solver["Procedure"] = [File("ResultOutputSolve"), File("ResultOutputSolver")]
    solver["Output Directory"] = File("Results")
    solver["Output File Name"] = File("case")
    solver["Vtu Format"] = True
    # solver["Ascii Output"] = True  # to save vtu as ascii
    solver["Save Geometry Ids"] = True
    solver["Displace Mesh"] = True
    solver["Single Precision"] = True  # default: False, False == 32 bit, True == 64 bit
    solver_list.append(solver)

    # solver - save statistical, integral, min, max, mean, ... values
    i += 1
    solver = Section(section="Solver", id=i)
    solver["Exec Solver"] = "After Simulation"
    solver["Equation"] = "SaveScalars"
    solver["Procedure"] = [File("SaveData"), File("SaveScalars")]
    solver["Output Directory"] = File("Results")
    solver["Filename"] = File("Scalars.dat")
    solver["Operator 1"] = "boundary int"
    solver["Variable 1"] = "Principal Stress 1"
    solver["Mask Name 1"] = "MASTER_ROTOR_BOUNDARY"
    solver["Operator 2"] = "boundary int"
    solver["Variable 2"] = "Principal Stress 2"
    solver["Mask Name 2"] = "MASTER_ROTOR_BOUNDARY"
    solver["Operator 3"] = "boundary int"
    solver["Variable 3"] = "Principal Stress 1"
    solver["Mask Name 3"] = "SLAVE_ROTOR_BOUNDARY"
    solver["Operator 4"] = "boundary int"
    solver["Variable 4"] = "Principal Stress 2"
    solver["Mask Name 4"] = "SLAVE_ROTOR_BOUNDARY"
    solver_list.append(solver)

    # solver - save line values
    # if no variable given, all variables are saved
    i += 1
    solver = Section(section="Solver", id=i)
    solver["Exec Solver"] = "After Simulation"
    solver["Equation"] = "SaveLineValues"
    solver["Procedure"] = [File("SaveData"), File("SaveLine")]
    solver["Output Directory"] = File("Results")
    solver["Filename"] = File("LineValues.dat")
    solver["Variable 1"] = "Displacement 1"
    solver["Variable 2"] = "Displacement 2"
    solver["Variable 3"] = "Principal Stress 1"
    solver["Variable 4"] = "Principal Stress 2"
    solver["Variable 5"] = "VonMises"
    # solver["Save Mask 1"] = "MASTER_ROTOR_BOUNDARY"
    # solver["Skip Boundary Info"] = Logical True
    solver_list.append(solver)

    # Variable 1 = Stress Bodyforce 1
    # Mask Name 1 = BodyForce
    # Operator 2 = boundary int
    # Variable 2 = Displacement Loads 1
    # Mask Name 2 = Top_0
    # Operator 3 = boundary int
    # Variable 3 = Displacement Loads 2
    # Mask Name 3 = Top_0
    # Operator 4 = boundary int
    # Variable 4 = Displacement Contact Load 1
    # Mask Name 4 = Top_0

    # Solver 5
    # Equation = "SaveMaterial"
    # Procedure = File "SaveData" "SaveMaterials"
    # !Parameter 1 = String "Stress Bodyforce 1"
    # !Parameter 2 = String "Stress Bodyforce 2"
    # !Body Force Parameters = 1

    # --- equations ------------------------------------------------------------------ #
    eqs = Section(section="Equation", id=1)
    eqs["Name"] = "Equation"
    eqs["Active Solvers"] = [i + 1 for i in range(len(solver_list))]

    # --- materials ------------------------------------------------------------------ #
    # TODO get magnets materials is inconvienent
    materials = [None, None, None]  # one extra element since ID_xx starts with 1

    mat_section = [Section(section="Material", id=ID_LAM)]
    materials[ID_LAM] = output.simu.machine.rotor.mat_type

    if self.include_magnets:
        mat_section.append(Section(section="Material", id=ID_MAG))
        materials[ID_MAG] = output.simu.machine.rotor.hole[0].magnet_0.mat_type

    # add materials to material sections
    for idx in range(len(mat_section)):
        ID = mat_section[idx].id
        mat_section[idx]["Name"] = materials[ID].name
        mat_section[idx]["Density"] = float(materials[ID].struct.rho)
        mat_section[idx]["Youngs modulus"] = float(materials[ID].struct.Ex)
        mat_section[idx]["Poisson ratio"] = float(materials[ID].struct.nu_xy)
        # TODO check anisotropy

    # --- boundary conditions -------------------------------------------------------- #

    boundaries = []
    i = 0

    # pair master and slave of Magnet_x_Top, Magnet_0_Right and Magnet_1_Left
    paired_bnds = [
        "Rotor_Magnet_0_Top",
        "Rotor_Magnet_1_Top",
        "Rotor_Magnet_0_Right",
        "Rotor_Magnet_1_Left",
    ]

    for name in paired_bnds:
        master = name + "_Master"
        slave = name + "_Slave"
        if master in names and slave in names:
            # "slave"
            i += 1
            bnd = Section(section="Boundary Condition", id=i)
            bnd["Name"] = slave
            bnd["Normal-Tangential Displacement"] = True
            bnd["Periodic BC"] = i + 1  # next bnd will be the corresponding master
            bnd["Periodic BC Displacement 1"] = True  # normal disp is fixed between M&S
            bnd["Periodic BC Displacement 2"] = False  # tangential disp is independent
            bnd["Periodic BC Pressure"] = False  # pressure can be independent
            bnd["Top_0"] = True  # for save values
            boundaries.append(bnd)

            # "master"
            i += 1
            bnd = Section(section="Boundary Condition", id=i)
            bnd["Name"] = master
            bnd["Normal-Tangential Displacement"] = True
            boundaries.append(bnd)

    # no normal displacement on lamination sides
    i += 1
    bnd = Section(section="Boundary Condition", id=i)
    bnd["Name"] = "MASTER_ROTOR_BOUNDARY"
    bnd["Displacement 1"] = 0.0
    bnd["Normal-Tangential Displacement"] = True
    bnd["Save Line"] = True
    bnd["Save Scalars"] = True
    bnd["MASTER_ROTOR_BOUNDARY"] = True  # mask name for SaveLine and SaveScalar
    boundaries.append(bnd)

    i += 1
    bnd = Section(section="Boundary Condition", id=i)
    bnd["Name"] = "SLAVE_ROTOR_BOUNDARY"
    bnd["Displacement 1"] = 0.0
    bnd["Normal-Tangential Displacement"] = True
    bnd["Save Line"] = True
    bnd["Save Scalars"] = True
    bnd["SLAVE_ROTOR_BOUNDARY"] = True  # mask name for SaveLine and SaveScalar
    boundaries.append(bnd)

    i += 1  # Boundary to save line values
    bnd = Section(section="Boundary Condition", id=i)
    bnd["Name"] = "ROTOR_BORE_CURVE"
    # bnd["Normal-Tangential Displacement"] = True
    bnd["Save Line"] = True
    bnd["Save Scalars"] = True
    boundaries.append(bnd)

    # --- body force ----------------------------------------------------------------- #
    omega = 2 * pi * self.parent.input.N0 / 60
    bfs = []

    ID_list = [
        ID_LAM,
    ]
    if self.include_magnets:
        ID_list.append(ID_MAG)

    for ID in ID_list:
        matc = MATC(expr=f"{materials[ID].struct.rho}*{omega}^2*tx(0)")
        bf = Section(section="Body Force", id=ID)
        bf["Name"] = "Stress"
        bf["Mesh Rotate 3"] = 0.0
        bf["Stress Bodyforce 1"] = Variable(name="Coordinate 1", value=matc)
        bf["Stress Bodyforce 2"] = Variable(name="Coordinate 2", value=matc)
        bfs.append(bf)

    # -------------------------------------------------------------------------------- #
    # list of section - list need to be extended, single obj. can be appended
    sect_list = []
    sect_list.extend(bodies)
    sect_list.append(sim)
    sect_list.append(const)
    sect_list.extend(mat_section)
    sect_list.extend(solver_list)
    sect_list.append(eqs)
    sect_list.extend(boundaries)
    sect_list.extend(bfs)

    # create SolverInputFile obj.
    sif = SolverInputFile(sections=sect_list)

    # save the sif file
    with open(sif_file, "wt") as f:
        sif.write(f)



# -*- coding: utf-8 -*-

from ....Classes.CondType21 import CondType21
from ....Classes.LamSlotWind import LamSlotWind
from ....Methods.Machine.LamSquirrelCage import *


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self : LamSquirrelCage
        A LamSquirrelCage object

    Returns
    -------
    None

    Raises
    _______
    SCC_NotARotor
        A LamSquirrelCage Lamination can't be a stator
    SCC_WrongCond
        A LamSquirrelCage's conductor must be a type 2_1
    """

    LamSlotWind.check(self)

    if self.is_stator:
        raise SCC_NotARotor("A LamSquirrelCage Lamination can't be " "a stator")

    if not isinstance(self.winding.conductor, CondType21):
        raise SCC_WrongCond("A LamSquirrelCage's conductor must be " "a type 2_1")



from ....Methods.Slot.Slot import SlotCheckError


class S60_InnerCheckError(SlotCheckError):
    """ """

    pass


class S60_W21CheckError(SlotCheckError):
    """ """

    pass


class S60_RCheckError(SlotCheckError):
    """ """

    pass


class S60_WindHError(SlotCheckError):
    """ """

    pass


class S60_WindWError(SlotCheckError):
    """ """

    pass


class S60_WindError(SlotCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    winding


    """

    pass



from ....Classes.SurfLine import SurfLine


def get_surfaces(self):
    """Convert surf_dict to a list of surfaces without lines"""

    surf_list = list()

    for key, value in self.surf_dict.items():
        surf_list.append(SurfLine(line_list=[], point_ref=key, label=value))

    return surf_list



# -*-- coding: utf-8 -*


def translate(self, Zt):
    """Translate the Trapeze

    Parameters
    ----------
    self : Trapeze
        a Trapeze object

    Zt : complex
        Complex value for translation

    Returns
    -------
    None
    """
    # check if the Trapeze is correct
    self.check()
    self.point_ref += Zt



from PySide2.QtCore import QSize
from PySide2.QtWidgets import QGroupBox, QLabel, QVBoxLayout, QWidget

from ......GUI import gui_option


class WBarOut(QGroupBox):
    """Setup of QGroupBox for output for Winding Slot"""

    def __init__(self, parent=None):
        """Initialize the widget"""
        # Main widget setup
        QGroupBox.__init__(self, parent)
        self.u = gui_option.unit
        self.setTitle(self.tr("Output"))
        self.setMinimumSize(QSize(200, 0))
        self.setObjectName("g_output")
        self.layout = QVBoxLayout(self)
        self.layout.setObjectName("layout")

        # The widget is composed of 3 QLabel in a vertical layout
        self.out_Sbar = QLabel(self)
        self.out_Sbar.setObjectName("out_Sbar")
        self.layout.addWidget(self.out_Sbar)

        self.out_Sslot = QLabel(self)
        self.out_Sslot.setObjectName("out_Sslot")
        self.layout.addWidget(self.out_Sslot)

        self.out_ratio = QLabel(self)
        self.out_ratio.setMinimumSize(QSize(140, 0))
        self.out_ratio.setObjectName("out_ratio")
        self.layout.addWidget(self.out_ratio)

    def comp_output(self):
        """Update the Output text with the computed values

        Parameters
        ----------
        self : WBarOut
            A WBarOut object
        """

        # For readibility
        obj = self.parent().machine.rotor.winding.conductor

        # Update Bar surface if possible
        txt_Sbar = self.tr("Sbar: ")
        try:
            Sbar = format(self.u.get_m2(obj.comp_surface_active()), ".4g")
            self.out_Sbar.setText(txt_Sbar + Sbar + " " + self.u.get_m2_name())
        except Exception:
            self.out_Sbar.setText(txt_Sbar + "?")
        # Update Slot surface if possible
        txt_Sslot = self.tr("Sslot: ")
        try:
            Sslot = format(
                self.u.get_m2(self.parent().machine.rotor.slot.comp_surface()), ".4g"
            )
            self.out_Sslot.setText(txt_Sslot + Sslot + " " + self.u.get_m2_name())
        except Exception:
            self.out_Sslot.setText(txt_Sslot + " ?")
        # Update ratio if possible
        txt_ratio = self.tr("Sbar / Sslot: ")
        try:
            ratio = "%.4g" % (float(Sbar) / float(Sslot) * 100)
            self.out_ratio.setText(txt_ratio + ratio + u" %")
        except Exception:
            self.out_ratio.setText(txt_ratio + "?")



# -*- coding: utf-8 -*-

from numpy import cos


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW12
        A SlotW12 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    Rbo = self.get_Rbo()

    # Computation of the arc height
    alpha = self.comp_angle_opening() / 2
    Harc = float(Rbo * (1 - cos(alpha)))

    if self.is_outwards():
        return self.H0 - Harc + self.R1 * 2 + self.H1 + self.R2
    else:
        return self.H0 + Harc + self.R1 * 2 + self.H1 + self.R2



from ....Methods.Slot.Slot import SlotCheckError


class S54_NoneError(SlotCheckError):
    """Raised when a propery of HoleM54 is None"""

    pass


class H54_W0CheckError(SlotCheckError):
    """ """

    pass


class H54_R1CheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch
from matplotlib.pyplot import axis, legend
from numpy import exp

from ....Functions.init_fig import init_fig
from ....definitions import config_dict
from ....Methods import ParentMissingError

MAGNET_COLOR = config_dict["PLOT"]["COLOR_DICT"]["MAGNET_COLOR"]


def plot(
    self, fig=None, ax=None, display_magnet=True, is_add_arrow=False, is_show_fig=True
):
    """Plot the Hole in a matplotlib fig

    Parameters
    ----------
    self : Hole
        A Hole object
    fig :
        if None, open a new fig and plot, else add to the current
        one (Default value = None)
    display_magnet : bool
        if True, plot the magnet inside the hole, if there is any (Default value = True)
    is_add_arrow : bool
        To add an arrow for the magnetization

    Returns
    -------
    None
    """
    display = fig is None
    if display:
        color = "k"
    else:
        color = "w"

    surf_hole = self.build_geometry()
    patches = list()
    for surf in surf_hole:
        if "Magnet" in surf.label and display_magnet:
            patches.extend(surf.get_patches(color=MAGNET_COLOR))
        else:
            patches.extend(surf.get_patches(color=color))

    # Display the result
    (fig, axes, patch_leg, label_leg) = init_fig(fig, ax)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Hole")

    # Add all the hole (and magnet) to fig
    for patch in patches:
        axes.add_patch(patch)

    # Magnetization
    if is_add_arrow:
        H = self.comp_height()
        mag_dict = self.comp_magnetization_dict()
        for magnet_name, mag_dir in mag_dict.items():
            # Get the correct surface
            mag_surf = None
            mag_id = magnet_name.split("_")[-1]
            for surf in surf_hole:
                if "Magnet" in surf.label and "_T" + mag_id in surf.label:
                    mag_surf = surf
                    break
            # Create arrow coordinates
            Z1 = mag_surf.point_ref
            Z2 = mag_surf.point_ref + H / 5 * exp(1j * mag_dir)
            axes.annotate(
                text="",
                xy=(Z2.real, Z2.imag),
                xytext=(Z1.real, Z1.imag),
                arrowprops=dict(arrowstyle="->", linewidth=1, color="b"),
            )

    # Axis Setup
    axes.axis("equal")
    try:
        Lim = self.get_Rbo() * 1.2
        axes.set_xlim(-Lim, Lim)
        axes.set_ylim(-Lim, Lim)
    except ParentMissingError:
        pass

    if display_magnet and "Magnet" in [surf.label for surf in surf_hole]:
        patch_leg.append(Patch(color=MAGNET_COLOR))
        label_leg.append("Magnet")
        legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()



from ....Methods.Slot.Slot import SlotCheckError


def check(self):
    """Check that the SlotDC object is correct

    Parameters
    ----------
    self : SlotDC
        A SlotDC object

    Returns
    -------
    None
    """
    if self.W1 >= self.D1:
        raise SlotCheckError("Slot 17 must have W1 < D1")
    if self.W2 >= self.D1:
        raise SlotCheckError("Slot 17 must have W2 < D1")
    if self.W2 >= self.D2:
        raise SlotCheckError("Slot 17 must have W2 < D2")
    if self.H2 <= (self.D1 / 2 + self.D2 / 2):
        raise SlotCheckError("Slot 17 must have D1/2 + D2/2 < H2")



# -*- coding: utf-8 -*-

from numpy import arcsin, cos, exp, pi, sin, sqrt, tan


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    point_dict = self._comp_point_coordinate()
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]

    # Compute area of triangle Z5,Z6
    S1 = abs(Z6.imag) * abs(Z5.real - Z4.real) / 2
    S2 = (abs(Z6.imag) + abs(Z7.imag)) * abs(Z7.real - Z6.real) / 2

    if self.is_outwards():
        return (S1 + S2) * 2
    else:
        return (S2 - S1) * 2



# -*- coding: utf-8 -*-

from numpy import arcsin, cos


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW26
        A SlotW26 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """
    Rbo = self.get_Rbo()

    # Computation of the arc height (P1,0,P8)
    alpha = self.comp_angle_opening() / 2
    Harc = float(Rbo * (1 - cos(alpha)))

    # Height of the arc (P2,C1,P7)
    Hwind = self.comp_height_active()

    if self.is_outwards():
        return self.H0 + Hwind - Harc
    else:
        return self.H0 + Hwind + Harc



from numpy import pi, all as np_all


def comp_force(self, output, axes_dict):
    """Compute the air-gap surface force based on Maxwell Tensor (MT).

    Parameters
    ----------
    self : ForceMT
        A ForceMT object
    output : Output
        an Output object (to update)
    axes_dict: {Data}
        Dict of axes used for force calculation

    Returns
    -------
    out_dict: dict
        Dict containing the following quantities:
            AGSF_r : ndarray
                Airgap radial Maxwell stress (Nt,Na,Nz) [N/m²]
            AGSF_t : ndarray
                Airgap tangential Maxwell stress (Nt,Na,Nz) [N/m²]
            AGSF_z : ndarray
                Airgap axial Maxwell stress (Nt,Na,Nz) [N/m²]

    """

    # Init output dict
    out_dict = dict()

    Rag = output.mag.Rag
    out_dict["Rag"] = Rag

    # Get time and angular axes
    Angle = axes_dict["Angle"]
    Time = axes_dict["Time"]

    # Import angular vector from Angle Data object
    is_periodicity_a, is_antiper_a = Angle.get_periodicity()

    if self.is_periodicity_a is not None:
        is_periodicity_a = self.is_periodicity_a

    angle = Angle.get_values(
        is_oneperiod=is_periodicity_a,
        is_antiperiod=is_antiper_a and is_periodicity_a,
    )

    if self.is_periodicity_t is not None:
        is_periodicity_t = self.is_periodicity_t

    # Import time vector from Time Data object
    is_periodicity_t, is_antiper_t = Time.get_periodicity()
    time = Time.get_values(
        is_oneperiod=is_periodicity_t,
        is_antiperiod=is_antiper_t and is_periodicity_t,
    )

    # Load magnetic flux
    Brphiz = output.mag.B.get_rphiz_along(
        "time=axis_data",
        "angle=axis_data",
        axis_data={"time": time, "angle": angle},
    )
    Br = Brphiz["radial"]

    # Magnetic void permeability
    mu_0 = 4 * pi * 1e-7

    # Get flux density component lists
    comp_list = list(output.mag.B.components.keys())

    # Calculate Maxwell Stress Tensor
    if "radial" in comp_list:
        if "tangential" not in comp_list and "axial" not in comp_list:
            out_dict["AGSF_r"] = -Br * Br / (2 * mu_0)

        elif "tangential" in comp_list:
            Bt = Brphiz["tangential"]
            out_dict["AGSF_t"] = -Br * Bt / mu_0

            if "axial" not in comp_list:
                out_dict["AGSF_r"] = -(Br * Br - Bt * Bt) / (2 * mu_0)

            else:
                Bz = Brphiz["axial"]
                out_dict["AGSF_r"] = -(Br * Br - Bt * Bt - Bz * Bz) / (2 * mu_0)
                out_dict["AGSF_z"] = -Br * Bz / mu_0

    return out_dict



from ....Functions.Geometry.comp_flower_arc import comp_flower_arc
from ....Classes.Arc1 import Arc1
from ....Methods import ParentMissingError
from numpy import pi, exp


def get_bore_line(self, label=None):
    """Return the bore line description

    Parameters
    ----------
    self : BoreUD
        A BoreUD object

    Returns
    -------
    bore_list : list
        List of bore lines
    """

    if label is not None:
        for line in self.line_list:
            line.label = label

    return self.line_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/Interpolation/RefQuad9.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/RefQuad9
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .RefCell import RefCell

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.RefQuad9.shape_function import shape_function
except ImportError as error:
    shape_function = error

try:
    from ..Methods.Mesh.RefQuad9.jacobian import jacobian
except ImportError as error:
    jacobian = error

try:
    from ..Methods.Mesh.RefQuad9.grad_shape_function import grad_shape_function
except ImportError as error:
    grad_shape_function = error

try:
    from ..Methods.Mesh.RefQuad9.get_real_point import get_real_point
except ImportError as error:
    get_real_point = error

try:
    from ..Methods.Mesh.RefQuad9.get_ref_point import get_ref_point
except ImportError as error:
    get_ref_point = error

try:
    from ..Methods.Mesh.RefQuad9.is_inside import is_inside
except ImportError as error:
    is_inside = error


from ._check import InitUnKnowClassError


class RefQuad9(RefCell):
    """Store quadrangel elements for 2D mesh"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Mesh.RefQuad9.shape_function
    if isinstance(shape_function, ImportError):
        shape_function = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use RefQuad9 method shape_function: " + str(shape_function)
                )
            )
        )
    else:
        shape_function = shape_function
    # cf Methods.Mesh.RefQuad9.jacobian
    if isinstance(jacobian, ImportError):
        jacobian = property(
            fget=lambda x: raise_(
                ImportError("Can't use RefQuad9 method jacobian: " + str(jacobian))
            )
        )
    else:
        jacobian = jacobian
    # cf Methods.Mesh.RefQuad9.grad_shape_function
    if isinstance(grad_shape_function, ImportError):
        grad_shape_function = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use RefQuad9 method grad_shape_function: "
                    + str(grad_shape_function)
                )
            )
        )
    else:
        grad_shape_function = grad_shape_function
    # cf Methods.Mesh.RefQuad9.get_real_point
    if isinstance(get_real_point, ImportError):
        get_real_point = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use RefQuad9 method get_real_point: " + str(get_real_point)
                )
            )
        )
    else:
        get_real_point = get_real_point
    # cf Methods.Mesh.RefQuad9.get_ref_point
    if isinstance(get_ref_point, ImportError):
        get_ref_point = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use RefQuad9 method get_ref_point: " + str(get_ref_point)
                )
            )
        )
    else:
        get_ref_point = get_ref_point
    # cf Methods.Mesh.RefQuad9.is_inside
    if isinstance(is_inside, ImportError):
        is_inside = property(
            fget=lambda x: raise_(
                ImportError("Can't use RefQuad9 method is_inside: " + str(is_inside))
            )
        )
    else:
        is_inside = is_inside
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(self, epsilon=0.05, init_dict=None, init_str=None):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "epsilon" in list(init_dict.keys()):
                epsilon = init_dict["epsilon"]
        # Set the properties (value check and convertion are done in setter)
        # Call RefCell init
        super(RefQuad9, self).__init__(epsilon=epsilon)
        # The class is frozen (in RefCell init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        RefQuad9_str = ""
        # Get the properties inherited from RefCell
        RefQuad9_str += super(RefQuad9, self).__str__()
        return RefQuad9_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from RefCell
        if not super(RefQuad9, self).__eq__(other):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from RefCell
        diff_list.extend(super(RefQuad9, self).compare(other, name=name))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from RefCell
        S += super(RefQuad9, self).__sizeof__()
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from RefCell
        RefQuad9_dict = super(RefQuad9, self).as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        RefQuad9_dict["__class__"] = "RefQuad9"
        return RefQuad9_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        # Set to None the properties inherited from RefCell
        super(RefQuad9, self)._set_None()



# -*- coding: utf-8 -*-
from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1


def build_geometry(self):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotM17
        A SlotM17 object

    Returns
    -------
    curve_list: list
        Empty list (no lamination, only active surface)

    """

    return list()



# -*-- coding: utf-8 -*
def comp_length(self):
    """Returns the length of the Trapeze

    Parameters
    ----------
    self : Trapeze
        a Trapeze object


    Returns
    -------
    length: float
        Length of the surface [m]

    """
    # check if the Trapeze is correct
    self.check()
    lines = self.get_lines()
    length = 0
    for line in lines:
        length += line.comp_length()
    return length



# -*- coding: utf-8 -*-


def get_lam_by_label(self, label):
    """Returns the lamination by its labels. Accepted labels are 'Stator_X' and
    'Rotor_X' with X the number of the lamination starting with 0.
    For convenience also 'Stator' or 'Rotor' are allowed here to get respective first
    stator or rotor lamination.

    Parameters
    ----------
    self : Machine
        Machine object
    label : string
        label of the lamination to return

    Returns
    -------
    lam : Lamination
        Lamination with the given label
    """
    # prepare error message just in case
    err_msg = (
        f"'{label}' is not a valid input argument for label."
        + " Only 'Stator_X' or 'Rotor_X', with X the index, are accepted."
    )

    # split the label components
    lam_str = label.split("_")

    # if no index is provided append index 0
    if len(lam_str) == 1:
        label += "_0"

    # check that 'Rotor' or 'Stator' is in label
    if lam_str[0] not in ["Stator", "Rotor"]:
        raise LabelInputError(err_msg)

    # get the requested lamination index
    try:
        index = self.get_lam_list_label().index(label)
    except:
        raise LabelInputError(err_msg)

    # return requested lamination
    return self.get_lam_list(is_int_to_ext=True, key=None)[index]


class LabelInputError(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-

from numpy import arcsin, exp, pi, sqrt


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotW28
        A SlotW28 object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    point_dict = self._comp_point_coordinate()
    Z2 = point_dict["Z2"]
    Z5 = point_dict["Z5"]
    Z7 = point_dict["Z7"]
    if self.is_outwards():
        Ztan2 = Z5 + Z5.imag * (-1 - 1j)
    else:
        Ztan2 = Z5 + Z5.imag * (+1 - 1j)
    Ztan1 = (Z2 + Z7) / 2.0

    return abs(Ztan2 - Ztan1)



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch, Rectangle
from matplotlib.pyplot import axis, legend, subplots
from ....definitions import config_dict

COND_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][0].copy()
INS_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"][1].copy()
# Remove alpha from phases
COND_COLOR[3] = 1
INS_COLOR[3] = 1


def plot(self, is_show_fig=True):
    """Plot a Conductor in a matplotlib fig

    Parameters
    ----------
    self : CondType11
        A CondType11 object
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None
    """

    patches_list = []

    # Conductor insulation
    Wcond = self.comp_width()
    Hcond = self.comp_height()
    patches_list.append(Rectangle((0, 0), Wcond, Hcond, color=INS_COLOR))

    # Wire conductor
    for ii in range(self.Nwppc_tan):
        for jj in range(self.Nwppc_rad):
            # Computation of bottom left corner coodinates
            x = self.Wins_wire + ii * (self.Wwire + 2 * self.Wins_wire)
            y = self.Wins_wire + jj * (self.Hwire + 2 * self.Wins_wire)
            patches_list.append(
                Rectangle((x, y), self.Wwire, self.Hwire, color=COND_COLOR)
            )

    # Display
    fig, ax = subplots()
    for patch in patches_list:
        ax.add_patch(patch)

    # Axis Setup
    axis("equal")

    # The conductor is centered
    ax.set_xlim(0 - Wcond / 10, Wcond * 11.0 / 10.0)
    ax.set_ylim(0 - Hcond / 10, Hcond * 11.0 / 10.0)

    # Legend
    patch_leg = list()  # Symbol
    label_leg = list()  # Text
    if self.Wins_wire > 0:
        patch_leg.append(Patch(color=INS_COLOR))
        label_leg.append("Wire insulation")

    patch_leg.append(Patch(color=COND_COLOR))
    label_leg.append("Active wire section")

    legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-
from ....Functions.Simulation.create_from_axis import create_from_axis


def comp_axes(self, output):
    """Compute the axes required in any Magnetics module

    Parameters
    ----------
    self : Magnetic
        a Magnetic object
    output : Output
        an Output object (to update)

    Returns
    -------
    axes_dict: {Data}
        Dict containing Time and Angle axes including (anti-)periodicties used in any Magnetics module

    """

    # Store Time axis in OutMag
    output.mag.Time = output.elec.Time.copy()

    # Store Angle axis in OutMag
    output.mag.Angle = output.elec.Angle.copy()

    # Calculate axes for Magnetics module calculation
    # Get time and space (anti-)periodicities of the machine
    (
        per_a,
        is_antiper_a,
        per_t,
        is_antiper_t,
    ) = output.get_machine_periodicity()

    # Compute Time axis based on the one stored in OutElec
    Time, is_periodicity_t = create_from_axis(
        axis_in=output.mag.Time,
        per=per_t,
        is_aper=is_antiper_t,
        is_include_per=self.is_periodicity_t,
        is_remove_aper=False,
    )

    if is_periodicity_t != self.is_periodicity_t:
        # Remove time periodicity in Magnetic model
        self.is_periodicity_t = False
        Nt_tot = Time.get_length(is_oneperiod=False)
        self.get_logger().warning(
            "WARNING: In Magnetic model, Nt_tot="
            + str(Nt_tot)
            + " is not divisible by the machine time periodicity ("
            + str(per_t)
            + "). Time periodicity removed"
        )

    # Compute Angle axis based on the one stored in OutElec
    Angle, is_periodicity_a = create_from_axis(
        axis_in=output.mag.Angle,
        per=per_a,
        is_aper=is_antiper_a,
        is_include_per=self.is_periodicity_a,
        is_remove_aper=False,
    )

    if is_periodicity_a != self.is_periodicity_a:
        # Remove time periodicity in Magnetic model
        self.is_periodicity_a = False
        Na_tot = Angle.get_length(is_oneperiod=False)
        self.get_logger().warning(
            "WARNING: In Magnetic model, Na_tot="
            + str(Na_tot)
            + " is not divisible by the machine angular periodicity ("
            + str(per_a)
            + "). Angular periodicity removed"
        )

    # Add Time axis on which to calculate torque
    # Copy from standard Time axis
    Time_Tem = Time.copy()

    # Remove anti-periodicity if any
    if "antiperiod" in Time_Tem.symmetries:
        Time_Tem.symmetries["period"] = Time_Tem.symmetries.pop("antiperiod")

    # Store in axis dict
    axes_dict = {"Time": Time, "Angle": Angle, "Time_Tem": Time_Tem}

    return axes_dict



# -*- coding: utf-8 -*-


def get_rot_dir(self):
    """Return the rotation direction of the magnetic field fundamental
    WARNING: rot_dir = -1 to have positive rotor rotating direction, i.e. rotor position moves towards positive angle

    Parameters
    ----------
    self : Output
        an Output object

    Returns
    -------
    rot_dir: int
        Rotation direction of magnetic field fundamental

    """

    # Already available => Return
    if self.geo.rot_dir is not None:
        return self.geo.rot_dir
    # check for imposed rot_dir in Simulation
    elif (
        self.simu is not None
        and self.simu.input is not None
        and hasattr(self.simu.input, "rot_dir")
        and self.simu.input.rot_dir is not None
    ):
        rot_dir = self.simu.input.rot_dir
    else:  # Compute from stator winding
        rot_dir = self.simu.machine.stator.comp_rot_dir()

    self.geo.rot_dir = rot_dir
    return rot_dir



# -*- coding: utf-8 -*-
from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine
from ....Methods import ParentMissingError
from numpy import exp, pi


def get_surface_tooth(self):
    """Returns the surface delimiting the tooth (including yoke part)

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    surface: SurfLine
        A SurfLine object representing the slot

    """

    if self.parent is not None:
        Ryoke = self.parent.get_Ryoke()
    else:
        raise ParentMissingError("Error: The slot is not inside a Lamination")

    curve_list = list()
    # tooth lines
    top_list = self.build_geometry_half_tooth(is_top=True)
    bot_list = self.build_geometry_half_tooth(is_top=False)
    # Yoke lines
    Z1 = Ryoke * exp(1j * pi / self.Zs)
    Z2 = Ryoke * exp(-1j * pi / self.Zs)
    curve_list.append(Segment(top_list[-1].get_end(), Z1, label="Tooth_Yoke_Side_Left"))
    if Ryoke > 0:
        curve_list.append(
            Arc1(
                begin=Z1,
                end=Z2,
                radius=-Ryoke,
                is_trigo_direction=False,
                label="Tooth_Yoke_Arc",
            )
        )
    curve_list.append(
        Segment(Z2, bot_list[0].get_begin(), label="Tooth_Yoke_Side_Right")
    )
    # Second half of the tooth
    curve_list.extend(bot_list)
    curve_list.extend(top_list)

    return SurfLine(line_list=curve_list, label="Tooth")



# -*- coding: utf-8 -*-

# File generated according to DAVDuct.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from ......GUI.Tools.HelpButton import HelpButton

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_DAVDuct(object):
    def setupUi(self, DAVDuct):
        if not DAVDuct.objectName():
            DAVDuct.setObjectName(u"DAVDuct")
        DAVDuct.resize(767, 630)
        self.verticalLayout = QVBoxLayout(DAVDuct)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.b_new = QPushButton(DAVDuct)
        self.b_new.setObjectName(u"b_new")

        self.horizontalLayout_2.addWidget(self.b_new)

        self.b_remove = QPushButton(DAVDuct)
        self.b_remove.setObjectName(u"b_remove")

        self.horizontalLayout_2.addWidget(self.b_remove)

        self.b_help = HelpButton(DAVDuct)
        self.b_help.setObjectName(u"b_help")
        self.b_help.setPixmap(QPixmap(u":/images/images/icon/help_16.png"))

        self.horizontalLayout_2.addWidget(self.b_help)

        self.horizontalSpacer_2 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer_2)

        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.tab_vent = QTabWidget(DAVDuct)
        self.tab_vent.setObjectName(u"tab_vent")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.tab_vent.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName(u"tab_2")
        self.tab_vent.addTab(self.tab_2, "")

        self.verticalLayout.addWidget(self.tab_vent)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalSpacer_5 = QSpacerItem(
            218, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_4.addItem(self.horizontalSpacer_5)

        self.b_plot = QPushButton(DAVDuct)
        self.b_plot.setObjectName(u"b_plot")

        self.horizontalLayout_4.addWidget(self.b_plot)

        self.b_cancel = QPushButton(DAVDuct)
        self.b_cancel.setObjectName(u"b_cancel")

        self.horizontalLayout_4.addWidget(self.b_cancel)

        self.b_ok = QPushButton(DAVDuct)
        self.b_ok.setObjectName(u"b_ok")

        self.horizontalLayout_4.addWidget(self.b_ok)

        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.retranslateUi(DAVDuct)

        QMetaObject.connectSlotsByName(DAVDuct)

    # setupUi

    def retranslateUi(self, DAVDuct):
        DAVDuct.setWindowTitle(
            QCoreApplication.translate("DAVDuct", u"Set axial ventilation duct", None)
        )
        self.b_new.setText(QCoreApplication.translate("DAVDuct", u"Add New Set", None))
        self.b_remove.setText(
            QCoreApplication.translate("DAVDuct", u"Remove Last Set", None)
        )
        self.b_help.setText("")
        self.tab_vent.setTabText(
            self.tab_vent.indexOf(self.tab),
            QCoreApplication.translate("DAVDuct", u"Tab 1", None),
        )
        self.tab_vent.setTabText(
            self.tab_vent.indexOf(self.tab_2),
            QCoreApplication.translate("DAVDuct", u"Tab 2", None),
        )
        self.b_plot.setText(QCoreApplication.translate("DAVDuct", u"Preview", None))
        self.b_cancel.setText(QCoreApplication.translate("DAVDuct", u"Cancel", None))
        self.b_ok.setText(QCoreApplication.translate("DAVDuct", u"Ok", None))

    # retranslateUi



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Elmer/ElmerResultsVTU.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Elmer/ElmerResultsVTU
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Elmer import Elmer

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Elmer.ElmerResultsVTU.build_meshsolution import build_meshsolution
except ImportError as error:
    build_meshsolution = error


from ._check import InitUnKnowClassError


class ElmerResultsVTU(Elmer):
    """Class to get Elmer simulation results from a VTU file"""

    VERSION = 1

    # cf Methods.Elmer.ElmerResultsVTU.build_meshsolution
    if isinstance(build_meshsolution, ImportError):
        build_meshsolution = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ElmerResultsVTU method build_meshsolution: "
                    + str(build_meshsolution)
                )
            )
        )
    else:
        build_meshsolution = build_meshsolution
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        label="ElmerResults",
        file_path="",
        store_dict=-1,
        logger_name="Pyleecan.Elmer",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
            if "file_path" in list(init_dict.keys()):
                file_path = init_dict["file_path"]
            if "store_dict" in list(init_dict.keys()):
                store_dict = init_dict["store_dict"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.label = label
        self.file_path = file_path
        self.store_dict = store_dict
        # Call Elmer init
        super(ElmerResultsVTU, self).__init__(logger_name=logger_name)
        # The class is frozen (in Elmer init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ElmerResultsVTU_str = ""
        # Get the properties inherited from Elmer
        ElmerResultsVTU_str += super(ElmerResultsVTU, self).__str__()
        ElmerResultsVTU_str += 'label = "' + str(self.label) + '"' + linesep
        ElmerResultsVTU_str += 'file_path = "' + str(self.file_path) + '"' + linesep
        ElmerResultsVTU_str += "store_dict = " + str(self.store_dict) + linesep
        return ElmerResultsVTU_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Elmer
        if not super(ElmerResultsVTU, self).__eq__(other):
            return False
        if other.label != self.label:
            return False
        if other.file_path != self.file_path:
            return False
        if other.store_dict != self.store_dict:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Elmer
        diff_list.extend(super(ElmerResultsVTU, self).compare(other, name=name))
        if other._label != self._label:
            diff_list.append(name + ".label")
        if other._file_path != self._file_path:
            diff_list.append(name + ".file_path")
        if other._store_dict != self._store_dict:
            diff_list.append(name + ".store_dict")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Elmer
        S += super(ElmerResultsVTU, self).__sizeof__()
        S += getsizeof(self.label)
        S += getsizeof(self.file_path)
        if self.store_dict is not None:
            for key, value in self.store_dict.items():
                S += getsizeof(value) + getsizeof(key)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Elmer
        ElmerResultsVTU_dict = super(ElmerResultsVTU, self).as_dict(**kwargs)
        ElmerResultsVTU_dict["label"] = self.label
        ElmerResultsVTU_dict["file_path"] = self.file_path
        ElmerResultsVTU_dict["store_dict"] = (
            self.store_dict.copy() if self.store_dict is not None else None
        )
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ElmerResultsVTU_dict["__class__"] = "ElmerResultsVTU"
        return ElmerResultsVTU_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.label = None
        self.file_path = None
        self.store_dict = None
        # Set to None the properties inherited from Elmer
        super(ElmerResultsVTU, self)._set_None()

    def _get_label(self):
        """getter of label"""
        return self._label

    def _set_label(self, value):
        """setter of label"""
        check_var("label", value, "str")
        self._label = value

    label = property(
        fget=_get_label,
        fset=_set_label,
        doc=u"""Label of the resulting meshsolution

        :Type: str
        """,
    )

    def _get_file_path(self):
        """getter of file_path"""
        return self._file_path

    def _set_file_path(self, value):
        """setter of file_path"""
        check_var("file_path", value, "str")
        self._file_path = value

    file_path = property(
        fget=_get_file_path,
        fset=_set_file_path,
        doc=u"""Filename of the results VTU data file

        :Type: str
        """,
    )

    def _get_store_dict(self):
        """getter of store_dict"""
        return self._store_dict

    def _set_store_dict(self, value):
        """setter of store_dict"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("store_dict", value, "dict")
        self._store_dict = value

    store_dict = property(
        fget=_get_store_dict,
        fset=_set_store_dict,
        doc=u"""Dict containing the data names to store

        :Type: dict
        """,
    )



# -*- coding: utf-8 -*-


def get_middle(self):
    """Return the point at the middle of the Segment

    Parameters
    ----------
    self : Segment
        A Segment object

    Returns
    -------
    Zmid: complex
        Complex coordinates of the middle of the Segment
    """

    Z1 = self.begin
    Z2 = self.end

    Zmid = (Z1 + Z2) / 2.0
    # Return (0,0) if the point is too close from 0
    if abs(Zmid) < 1e-6:
        Zmid = 0
    return Zmid



# -*- coding: utf-8 -*-

from .plot_4D import plot_4D
from .plot_3D import plot_3D
from . import unit_dict, norm_dict, axes_dict
from numpy import where, meshgrid, max as np_max, min as np_min


def plot_3D_Data(
    data,
    *arg_list,
    is_norm=False,
    unit="SI",
    save_path=None,
    x_min=None,
    x_max=None,
    y_min=None,
    y_max=None,
    z_min=None,
    z_max=None,
    z_range=None,
    is_auto_ticks=True,
    is_2D_view=False,
    N_stem=100,
    fig=None,
    ax=None,
    is_show_fig=None,
):
    """Plots a field as a function of two axes

    Parameters
    ----------
    data : Data
        a Data object
    *arg_list : list of str
        arguments to specify which axes to plot
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    x_min : float
        minimum value for the x-axis
    x_max : float
        maximum value for the x-axis
    y_min : float
        minimum value for the y-axis
    y_max : float
        maximum value for the y-axis
    z_min : float
        minimum value for the z-axis
    z_max : float
        maximum value for the z-axis
    z_range : float
        range to use for the z-axis
    is_auto_ticks : bool
        in fft, adjust ticks to freqs (deactivate if too close)
    is_2D_view : bool
        True to plot Data in xy plane and put z as colormap
    N_stem : int
        number of harmonics to plot (only for stem plots)
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data
    is_show_fig : bool
        True to show figure after plot
    """

    if len(arg_list) == 1 and type(arg_list[0]) == tuple:
        arg_list = arg_list[0]  # if called from another script with *arg_list

    # Set unit
    if unit == "SI":
        unit = data.unit

    # Detect fft
    is_fft = False
    if any("wavenumber" in s for s in arg_list) or any("freqs" in s for s in arg_list):
        is_fft = True
        if "dB" in unit:
            unit_str = (
                "[" + unit + " re. " + str(data.normalizations["ref"]) + data.unit + "]"
            )
        else:
            unit_str = "[" + unit + "]"
        if data.symbol == "Magnitude":
            zlabel = "Magnitude " + unit_str
        else:
            zlabel = r"$|\widehat{" + data.symbol + "}|$ " + unit_str
        title1 = "FFT2 of " + data.name.lower() + " "
    else:
        if is_norm:
            zlabel = (
                r"$\frac{" + data.symbol + "}{" + data.symbol + "_0}\, [" + unit + "]$"
            )
        else:
            zlabel = r"$" + data.symbol + "\, [" + unit + "]$"
        title1 = "Surface plot of " + data.name.lower() + " "

    # Extract field and axes
    if is_fft:
        if is_2D_view:
            result = data.get_magnitude_along(arg_list, unit=unit, is_norm=is_norm)
        else:
            result = data.get_harmonics(
                N_stem, arg_list, unit=unit, is_norm=is_norm, is_flat=True
            )
    else:
        result = data.get_along(arg_list, unit=unit, is_norm=is_norm)
    axes_list = result["axes_list"]
    axes_dict_other = result["axes_dict_other"]
    Xdata = result[axes_list[0].name]
    Ydata = result[axes_list[1].name]
    Zdata = result[data.symbol]
    if is_fft and not is_2D_view:
        X_flat = Xdata
        Y_flat = Ydata
        Z_flat = Zdata

    else:
        Y_map, X_map = meshgrid(Ydata, Xdata)
        X_flat = X_map.flatten()
        Y_flat = Y_map.flatten()
        Z_flat = Zdata.flatten()
    if x_min is None:
        x_min = np_min(Xdata)
    if x_max is None:
        x_max = np_max(Xdata)
    if y_min is None:
        y_min = np_min(Ydata)
    if y_max is None:
        y_max = np_max(Ydata)
    if z_range is None:
        if z_min is None:
            z_min = np_min(Zdata)
        if z_max is None:
            z_max = np_max(Zdata)
    else:
        if z_min is None and z_max is None:
            z_max = np_max(Zdata)
        if z_max is None:
            z_max = z_min + z_range
        if z_min is None:
            z_min = z_max - z_range

    # Build labels and titles
    axis = axes_list[0]
    if axis.name in axes_dict:
        name = axes_dict[axis.name]
    else:
        name = axis.name
    title2 = "over " + axis.name.lower()
    if axis.unit == "SI":
        unit = unit_dict[axis.name]
        xlabel = name.capitalize() + " [" + unit + "]"
    elif axis.unit in norm_dict:
        xlabel = norm_dict[axis.unit]
    else:
        unit = axis.unit
        xlabel = name.capitalize() + " [" + unit + "]"
    if (
        axis.name == "angle"
        and axis.unit == "°"
        and round(np_max(axis.values) / 6) % 5 == 0
    ):
        xticks = [i * round(np_max(axis.values) / 6) for i in range(7)]
    else:
        xticks = None

    axis = axes_list[1]
    if axis.name in axes_dict:
        name = axes_dict[axis.name]
    else:
        name = axis.name
    title3 = " and " + axis.name.lower()
    if axis.unit == "SI":
        unit = unit_dict[axis.name]
        ylabel = name.capitalize() + " [" + unit + "]"
    elif axis.unit in norm_dict:
        ylabel = norm_dict[axis.unit]
    else:
        unit = axis.unit
        ylabel = name.capitalize() + " [" + unit + "]"
    if (
        axis.name == "angle"
        and axis.unit == "°"
        and round(np_max(axis.values) / 6) % 5 == 0
    ):
        yticks = [i * round(np_max(axis.values) / 6) for i in range(7)]
    else:
        yticks = None

    title4 = ""
    for axis in axes_list[2:]:
        if axis.unit == "SI":
            unit = unit_dict[axis.name]
        elif axis.unit in norm_dict:
            unit = norm_dict[axis.unit]
        else:
            unit = axis.unit
        title4 = " for " + axis.name + "=" + str(result[axis.name]) + " " + unit + ", "
    title5 = ""
    for axis_name in axes_dict_other:
        title5 += (
            axis_name
            + "="
            + str(axes_dict_other[axis_name][0])
            + " "
            + axes_dict_other[axis_name][1]
            + ", "
        )

    if title4 == " for " and title5 == "":
        title4 = ""

    title = title1 + title2 + title3 + title4 + title5
    title = title.rstrip(", ")

    # Call generic plot function
    if is_fft:
        if is_auto_ticks:
            indices = where(Z_flat > abs(0.01 * z_max))[0]
            xticks = X_flat[indices]
            yticks = Y_flat[indices]
        else:
            xticks = None
            yticks = None
        if is_2D_view:
            plot_4D(
                X_flat,
                Y_flat,
                Z_flat,
                is_same_size=True,
                z_max=z_max,
                z_min=z_min,
                title=title,
                xlabel=xlabel,
                ylabel=ylabel,
                zlabel=zlabel,
                fig=fig,
                ax=ax,
                type="scatter",
                save_path=save_path,
                is_show_fig=is_show_fig,
            )
        else:
            plot_3D(
                X_flat,
                Y_flat,
                Z_flat,
                fig=fig,
                ax=ax,
                x_min=x_min,
                x_max=x_max,
                y_min=y_max,
                y_max=y_min,
                z_min=z_min,
                z_max=z_max,
                title=title,
                xlabel=xlabel,
                ylabel=ylabel,
                zlabel=zlabel,
                type="stem",
                save_path=save_path,
                is_show_fig=is_show_fig,
            )
    else:
        if is_2D_view:
            plot_3D(
                X_map,
                Y_map,
                Zdata,
                z_max=z_max,
                z_min=z_min,
                xlabel=xlabel,
                ylabel=ylabel,
                zlabel=zlabel,
                title=title,
                xticks=xticks,
                fig=fig,
                ax=ax,
                type="pcolor",
                save_path=save_path,
                is_show_fig=is_show_fig,
            )
        else:
            plot_3D(
                X_map,
                Y_map,
                Zdata,
                fig=fig,
                ax=ax,
                x_min=x_min,
                x_max=x_max,
                y_min=y_min,
                y_max=y_max,
                z_min=z_min,
                z_max=z_max,
                title=title,
                xlabel=xlabel,
                ylabel=ylabel,
                zlabel=zlabel,
                yticks=yticks,
                type="surf",
                save_path=save_path,
                is_show_fig=is_show_fig,
            )



from ....Functions.Machine.get_material import get_material


def get_material_list(self):
    """
    Get the list of unique materials contained in a Machine

    Parameters
    ----------
    obj : Pyleecan object

    Returns
    -------
    materials : list of unique materials contained in the object
    """

    return get_material(self)



# -*- coding: utf-8 -*-

from numpy import pi


def comp_mass(self):
    """Compute the mass of the Frame

    Parameters
    ----------
    self : Frame
        A Frame object

    Returns
    -------
    Mfra: float
        Mass of the Frame [kg]

    """

    Vfra = self.comp_volume()

    # Mass computation
    if Vfra == 0:
        return 0
    else:
        return Vfra * self.mat_type.struct.rho



# -*- coding: utf-8 -*-


from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from .....GUI import gui_option
from .....GUI.Dialog.DMachineSetup.SBar.Gen_SBar import Gen_SBar
from .....GUI.Dialog.DMachineSetup.SBar.PCondType21.PCondType21 import PCondType21
from .....GUI.Dialog.DMachineSetup.SBar.PCondType22.PCondType22 import PCondType22

# Information to fill the conductor type combobox
WIDGET_LIST = [PCondType21, PCondType22]
INIT_INDEX = [wid.cond_type for wid in WIDGET_LIST]
COND_NAME = [wid.cond_name for wid in WIDGET_LIST]


class SBar(Gen_SBar, QWidget):
    """Step to setup the Rotor Bar for SCIM machine"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for the DMachineSetup nav
    step_name = "Bar"

    def __init__(self, machine, matlib, is_stator=False):
        """Initialize the widget according to machine

        Parameters
        ----------
        self : SBar
            A SBar widget
        machine : Machine
            current machine to edit
        matlib : MatLib
            Material Library
        is_stator : bool
            To adapt the GUI to set either the stator or the rotor
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Saving arguments
        self.machine = machine
        self.matlib = matlib
        self.is_stator = is_stator

        # Set FloatEdit unit
        self.lf_Hscr.unit = "m"
        self.lf_Lscr.unit = "m"
        self.lf_Lewout.unit = "m"
        # Set unit name (m ou mm)
        wid_list = [self.unit_Hscr, self.unit_Lscr, self.unit_Lewout]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")

        # Update winding qs
        if (
            self.machine.rotor.slot.Zs is not None
            and self.machine.rotor.winding.qs is None
        ):
            self.machine.rotor.winding.qs = self.machine.rotor.slot.Zs

        # Set materials
        self.w_mat.def_mat = "Copper1"
        self.w_mat.setText("Ring material:")
        self.w_mat.update(self.machine.rotor, "ring_mat", self.matlib)
        self.w_mat.b_matlib.hide()

        # Initialize the GUI with the current machine value
        self.lf_Hscr.setValue(machine.rotor.Hscr)
        self.lf_Lscr.setValue(machine.rotor.Lscr)
        self.lf_Lewout.setValue(machine.rotor.winding.Lewout)

        # Fill the combobox
        self.c_bar_type.clear()
        for cond in COND_NAME:
            self.c_bar_type.addItem(cond)
        # Initialize the needed conductor page
        conductor = machine.rotor.winding.conductor
        if type(conductor) in INIT_INDEX:
            index = INIT_INDEX.index(type(conductor))
            self.g_bar.layout().removeWidget(self.w_bar)
            self.w_bar.setParent(None)
            self.w_bar = WIDGET_LIST[index](self.machine, self.matlib)
            self.g_bar.layout().addWidget(self.w_bar)
            self.c_bar_type.setCurrentIndex(index)
        else:  # Set default conductor
            self.s_set_bar_type(0)
            self.c_bar_type.setCurrentIndex(0)
        self.w_bar.saveNeeded.connect(self.emit_save)

        # Connect the signal/slot
        self.lf_Hscr.editingFinished.connect(self.set_Hscr)
        self.lf_Lscr.editingFinished.connect(self.set_Lscr)
        self.lf_Lewout.editingFinished.connect(self.set_Lewout)
        self.c_bar_type.currentIndexChanged.connect(self.s_set_bar_type)
        self.b_plot.clicked.connect(self.s_plot)
        self.w_mat.saveNeeded.connect(self.emit_save)

    def emit_save(self):
        """Emit the saveNeeded signal"""
        self.saveNeeded.emit()

    def set_Hscr(self):
        """Signal to update the value of Hscr according to the line edit

        Parameters
        ----------
        self : SBar
            A SBar object
        """
        self.machine.rotor.Hscr = self.lf_Hscr.value()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Lscr(self):
        """Signal to update the value of Lscr according to the line edit

        Parameters
        ----------
        self : SBar
            A SBar object
        """
        self.machine.rotor.Lscr = self.lf_Lscr.value()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def set_Lewout(self):
        """Signal to update the value of Lewout according to the line edit

        Parameters
        ----------
        self : SBar
            A SBar object
        """
        self.machine.rotor.winding.Lewout = self.lf_Lewout.value()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def s_set_bar_type(self, index):
        """Setup the Gui for the selected conductor type

        Parameters
        ----------
        self : SBar
            A SBar object
        index : int
            Index of the selected conductor type
        """

        # Remove the old widget
        self.g_bar.layout().removeWidget(self.w_bar)
        self.w_bar.setParent(None)

        # Initialize the new widget and conductor
        self.machine.rotor.winding.conductor = INIT_INDEX[index]()
        self.machine.rotor.winding.conductor._set_None()

        self.w_bar = WIDGET_LIST[index](self.machine, self.matlib)
        self.w_bar.saveNeeded.connect(self.emit_save)
        # Refresh the GUi
        self.g_bar.layout().addWidget(self.w_bar)
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def s_plot(self):
        """Try to plot the machine

        Parameters
        ----------
        self : SBar
            A SBar object
        """
        self.machine.plot()

    @staticmethod
    def check(lamination):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lamination : Lamination
            Lamination to check

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        try:
            # Check that everything is set
            if lamination.Hscr is None:
                return "You must set Hscr !"
            elif lamination.Lscr is None:
                return "You must set Lscr !"
            elif lamination.winding.Lewout is None:
                return "You must set Lewout !"

            if type(lamination.winding.conductor) is INIT_INDEX[0]:
                if lamination.winding.conductor.Hbar is None:
                    return "You must set Hbar !"
                elif lamination.winding.conductor.Wbar is None:
                    return "You must set Wbar !"
        except Exception as e:
            return str(e)



# -*- coding: utf-8 -*-
import numpy as np
import copy

from pyleecan.Classes.CellMat import CellMat
from pyleecan.Classes.PointMat import PointMat


def renum(self):
    """Remove non-used points, restart point indices from 0, and update connectivity. Indices of cells stay the same.

    Parameters
    ----------
    self : MeshMat
        an Mesh object

    Returns
    -------

    """

    coord_init = self.point.coordinate
    connect_dict, nb_cell, indices = self.get_cell()

    node_indice = list()
    for key in connect_dict:
        node_indice.extend(np.unique(connect_dict[key]))

    node_indice = np.unique(node_indice)

    nb_node_new = len(node_indice)
    node_indice_new = np.linspace(0, nb_node_new - 1, nb_node_new, dtype=int)
    connect_dict_new = copy.deepcopy(connect_dict)
    for inode in range(nb_node_new):
        for key in connect_dict:
            connect_dict_new[key][
                connect_dict[key] == node_indice[inode]
            ] = node_indice_new[inode]

    self.point = PointMat(
        coordinate=coord_init[node_indice, :], nb_pt=nb_node_new, indice=node_indice_new
    )

    for key in connect_dict:
        self.cell[key] = CellMat(
            connectivity=connect_dict_new[key],
            nb_cell=len(connect_dict_new[key]),
            nb_pt_per_cell=self.cell[key].nb_pt_per_cell,
            indice=self.cell[key].indice,
            interpolation=self.cell[key].interpolation,
        )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Geometry/Arc3.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Geometry/Arc3
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Arc import Arc

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Geometry.Arc3.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Geometry.Arc3.comp_length import comp_length
except ImportError as error:
    comp_length = error

try:
    from ..Methods.Geometry.Arc3.comp_radius import comp_radius
except ImportError as error:
    comp_radius = error

try:
    from ..Methods.Geometry.Arc3.discretize import discretize
except ImportError as error:
    discretize = error

try:
    from ..Methods.Geometry.Arc3.get_angle import get_angle
except ImportError as error:
    get_angle = error

try:
    from ..Methods.Geometry.Arc3.get_begin import get_begin
except ImportError as error:
    get_begin = error

try:
    from ..Methods.Geometry.Arc3.get_center import get_center
except ImportError as error:
    get_center = error

try:
    from ..Methods.Geometry.Arc3.get_end import get_end
except ImportError as error:
    get_end = error

try:
    from ..Methods.Geometry.Arc3.get_middle import get_middle
except ImportError as error:
    get_middle = error

try:
    from ..Methods.Geometry.Arc3.reverse import reverse
except ImportError as error:
    reverse = error

try:
    from ..Methods.Geometry.Arc3.rotate import rotate
except ImportError as error:
    rotate = error

try:
    from ..Methods.Geometry.Arc3.scale import scale
except ImportError as error:
    scale = error

try:
    from ..Methods.Geometry.Arc3.split_half import split_half
except ImportError as error:
    split_half = error

try:
    from ..Methods.Geometry.Arc3.translate import translate
except ImportError as error:
    translate = error


from ._check import InitUnKnowClassError


class Arc3(Arc):
    """Half circle define by two points"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Geometry.Arc3.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Geometry.Arc3.comp_length
    if isinstance(comp_length, ImportError):
        comp_length = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method comp_length: " + str(comp_length))
            )
        )
    else:
        comp_length = comp_length
    # cf Methods.Geometry.Arc3.comp_radius
    if isinstance(comp_radius, ImportError):
        comp_radius = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method comp_radius: " + str(comp_radius))
            )
        )
    else:
        comp_radius = comp_radius
    # cf Methods.Geometry.Arc3.discretize
    if isinstance(discretize, ImportError):
        discretize = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method discretize: " + str(discretize))
            )
        )
    else:
        discretize = discretize
    # cf Methods.Geometry.Arc3.get_angle
    if isinstance(get_angle, ImportError):
        get_angle = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method get_angle: " + str(get_angle))
            )
        )
    else:
        get_angle = get_angle
    # cf Methods.Geometry.Arc3.get_begin
    if isinstance(get_begin, ImportError):
        get_begin = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method get_begin: " + str(get_begin))
            )
        )
    else:
        get_begin = get_begin
    # cf Methods.Geometry.Arc3.get_center
    if isinstance(get_center, ImportError):
        get_center = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method get_center: " + str(get_center))
            )
        )
    else:
        get_center = get_center
    # cf Methods.Geometry.Arc3.get_end
    if isinstance(get_end, ImportError):
        get_end = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method get_end: " + str(get_end))
            )
        )
    else:
        get_end = get_end
    # cf Methods.Geometry.Arc3.get_middle
    if isinstance(get_middle, ImportError):
        get_middle = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method get_middle: " + str(get_middle))
            )
        )
    else:
        get_middle = get_middle
    # cf Methods.Geometry.Arc3.reverse
    if isinstance(reverse, ImportError):
        reverse = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method reverse: " + str(reverse))
            )
        )
    else:
        reverse = reverse
    # cf Methods.Geometry.Arc3.rotate
    if isinstance(rotate, ImportError):
        rotate = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method rotate: " + str(rotate))
            )
        )
    else:
        rotate = rotate
    # cf Methods.Geometry.Arc3.scale
    if isinstance(scale, ImportError):
        scale = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method scale: " + str(scale))
            )
        )
    else:
        scale = scale
    # cf Methods.Geometry.Arc3.split_half
    if isinstance(split_half, ImportError):
        split_half = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method split_half: " + str(split_half))
            )
        )
    else:
        split_half = split_half
    # cf Methods.Geometry.Arc3.translate
    if isinstance(translate, ImportError):
        translate = property(
            fget=lambda x: raise_(
                ImportError("Can't use Arc3 method translate: " + str(translate))
            )
        )
    else:
        translate = translate
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        begin=0,
        end=0,
        is_trigo_direction=False,
        label="",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "begin" in list(init_dict.keys()):
                begin = init_dict["begin"]
            if "end" in list(init_dict.keys()):
                end = init_dict["end"]
            if "is_trigo_direction" in list(init_dict.keys()):
                is_trigo_direction = init_dict["is_trigo_direction"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
        # Set the properties (value check and convertion are done in setter)
        self.begin = begin
        self.end = end
        self.is_trigo_direction = is_trigo_direction
        # Call Arc init
        super(Arc3, self).__init__(label=label)
        # The class is frozen (in Arc init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Arc3_str = ""
        # Get the properties inherited from Arc
        Arc3_str += super(Arc3, self).__str__()
        Arc3_str += "begin = " + str(self.begin) + linesep
        Arc3_str += "end = " + str(self.end) + linesep
        Arc3_str += "is_trigo_direction = " + str(self.is_trigo_direction) + linesep
        return Arc3_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Arc
        if not super(Arc3, self).__eq__(other):
            return False
        if other.begin != self.begin:
            return False
        if other.end != self.end:
            return False
        if other.is_trigo_direction != self.is_trigo_direction:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Arc
        diff_list.extend(super(Arc3, self).compare(other, name=name))
        if other._begin != self._begin:
            diff_list.append(name + ".begin")
        if other._end != self._end:
            diff_list.append(name + ".end")
        if other._is_trigo_direction != self._is_trigo_direction:
            diff_list.append(name + ".is_trigo_direction")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Arc
        S += super(Arc3, self).__sizeof__()
        S += getsizeof(self.begin)
        S += getsizeof(self.end)
        S += getsizeof(self.is_trigo_direction)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Arc
        Arc3_dict = super(Arc3, self).as_dict(**kwargs)
        if self.begin is None:
            Arc3_dict["begin"] = None
        elif isinstance(self.begin, float):
            Arc3_dict["begin"] = self.begin
        else:
            Arc3_dict["begin"] = str(self.begin)
        if self.end is None:
            Arc3_dict["end"] = None
        elif isinstance(self.end, float):
            Arc3_dict["end"] = self.end
        else:
            Arc3_dict["end"] = str(self.end)
        Arc3_dict["is_trigo_direction"] = self.is_trigo_direction
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        Arc3_dict["__class__"] = "Arc3"
        return Arc3_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.begin = None
        self.end = None
        self.is_trigo_direction = None
        # Set to None the properties inherited from Arc
        super(Arc3, self)._set_None()

    def _get_begin(self):
        """getter of begin"""
        return self._begin

    def _set_begin(self, value):
        """setter of begin"""
        if isinstance(value, str):
            value = complex(value)
        check_var("begin", value, "complex")
        self._begin = value

    begin = property(
        fget=_get_begin,
        fset=_set_begin,
        doc=u"""begin point of the arc

        :Type: complex
        """,
    )

    def _get_end(self):
        """getter of end"""
        return self._end

    def _set_end(self, value):
        """setter of end"""
        if isinstance(value, str):
            value = complex(value)
        check_var("end", value, "complex")
        self._end = value

    end = property(
        fget=_get_end,
        fset=_set_end,
        doc=u"""end of the arc

        :Type: complex
        """,
    )

    def _get_is_trigo_direction(self):
        """getter of is_trigo_direction"""
        return self._is_trigo_direction

    def _set_is_trigo_direction(self, value):
        """setter of is_trigo_direction"""
        check_var("is_trigo_direction", value, "bool")
        self._is_trigo_direction = value

    is_trigo_direction = property(
        fget=_get_is_trigo_direction,
        fset=_set_is_trigo_direction,
        doc=u"""Rotation direction of the arc

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-


def set_FEMM_circuit_prop(femm, circuits, Clabel, I=None):
    """Create or update the property of a circuit

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    circuits: list
        list the name of all circuits
    Clabel: str
        the label of the circuit surface
    q_id : int
        Index of the phase
    I : ndarray
        Lamination currents for given time step and for all phases [qs,1]

    Returns
    -------
    circuits : list
        list the name of the circuits in FEMM

    """

    # Find phase index of current surface label
    q_id = int(Clabel[5:])

    if Clabel not in circuits:
        # Create new circuit
        femm.mi_addcircprop(Clabel, 0, 1)
        circuits.append(Clabel)

    # Update circuit
    if I is not None:
        femm.mi_modifycircprop(Clabel, 1, I[q_id])
    else:
        femm.mi_modifycircprop(Clabel, 1, 0)

    return circuits



# -*- coding: utf-8 -*-
"""File generated according to PVentCirc/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.PVentCirc.Ui_PVentCirc import (
    Ui_PVentCirc,
)


class Gen_PVentCirc(Ui_PVentCirc):
    def setupUi(self, PVentCirc):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PVentCirc.setupUi(self, PVentCirc)
        # Setup of in_Alpha0
        txt = self.tr(u"""Shift angle of the holes around circumference""")
        self.in_Alpha0.setWhatsThis(txt)
        self.in_Alpha0.setToolTip(txt)

        # Setup of lf_Alpha0
        self.lf_Alpha0.validator().setBottom(0)
        self.lf_Alpha0.validator().setTop(6.29)
        txt = self.tr(u"""Shift angle of the holes around circumference""")
        self.lf_Alpha0.setWhatsThis(txt)
        self.lf_Alpha0.setToolTip(txt)

        # Setup of in_D0
        txt = self.tr(u"""Hole diameters""")
        self.in_D0.setWhatsThis(txt)
        self.in_D0.setToolTip(txt)

        # Setup of lf_D0
        self.lf_D0.validator().setBottom(0)
        txt = self.tr(u"""Hole diameters""")
        self.lf_D0.setWhatsThis(txt)
        self.lf_D0.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Diameter of the hole centers""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Diameter of the hole centers""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)



# -*- coding: utf-8 -*-


def create_FEMM_bar(femm, is_mmfr, rho, materials):
    """Create the property for LamSquirrel cage on the rotor

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    is_mmfr : bool
        1 to compute the rotor magnetomotive force / rotor magnetic field
    rho : float
        the Resistivity at 20°C
    materials : list
        List of materials already created in FEMM

    Returns
    -------
    (str, list)
        material name "Rotor Bar", updated materials

    """

    if is_mmfr:
        if "Rotor Bar" not in materials:
            femm.mi_addmaterial(
                "Rotor Bar", 1, 1, 0, 0, 1e-6 / rho, 0, 0, 1, 0, 0, 0, 0, 0
            )
            materials.append("Rotor Bar")
    else:
        if "Rotor Bar" not in materials:
            # replacing the rotor bars by air
            femm.mi_addmaterial("Rotor Bar", 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0)
            materials.append("Rotor Bar")

    return "Rotor Bar", materials



# -*- coding: utf-8 -*-
from numpy import pi


def comp_surface_active(self):
    """Compute the Slot inner active surface (by analytical computation)

    Parameters
    ----------
    self : SlotM11
        A SlotM11 object

    Returns
    -------
    Swind: float
        Slot inner active surface [m**2]

    """

    point_dict = self._comp_point_coordinate()
    ZM1 = point_dict["ZM1"]
    ZM2 = point_dict["ZM2"]

    R1 = abs(ZM1)
    R2 = abs(ZM2)

    S1 = pi * R1 ** 2 * (self.Wmag / (2 * pi))
    S2 = pi * R2 ** 2 * (self.Wmag / (2 * pi))

    return abs(S1 - S2)



# -*- coding: utf-8 -*-


def get_field(self, *args):
    """Get the value of variables stored in Solution.

    Parameters
    ----------
    self : SolutionData
        an SolutionData object
    *args: list of strings
        List of axes requested by the user, their units and values (optional)

    Returns
    -------
    field: array
        an array of field values

    """
    axname, _ = self.get_axes_list()
    symbol = self.field.symbol

    if not args:
        field_dict = self.field.get_along(tuple(axname))
    else:
        field_dict = self.field.get_along(args)

    field = field_dict[symbol]

    return field



class VentilationTrapInstanceError(Exception):
    """Raised when a Ventilation Trap parameter is not int or float"""

    pass


class TrapezeBuildGeometryError(Exception):
    """raised when the parameter have not good type"""

    pass



def get_param_simu(self, idx):
    """
    Return the list of parameter values for a specific simulation
    """
    assert len(idx) == len(self.shape)

    # Create a list to store values
    param_value = []
    for pe in self.paramexplorer_list:
        param_value.append(pe.value[idx])

    return param_value



# -*- coding: utf-8 -*-
import numpy as np


def get_field(self, *args):
    """Get the value of variables stored in Solution.

    Parameters
    ----------
    self : SolutionVector
        an SolutionVector object
    *args: list of strings
        List of axes requested by the user, their units and values (optional)

    Returns
    -------
    field: array
        an array of field values

    """

    axname, axsize = self.get_axes_list()

    id = 0
    for name in axname:
        if name == "component":
            if axsize[id] == 2:
                dim = 2
            else:
                dim = 3
        id += 1

    if not args:
        field = np.zeros(axsize)
        field_dict = self.field.get_xyz_along(tuple(axname))
    else:
        field_dict = self.field.get_xyz_along(args)
        comp_x = field_dict["comp_x"]
        size = np.hstack((comp_x.shape, dim))
        field = np.zeros(size)

    field[..., 0] = field_dict["comp_x"]
    field[..., 1] = field_dict["comp_y"]

    if dim == 3:
        field[..., 2] = field_dict["comp_z"]

    return field



# -*- coding: utf-8 -*-

# File generated according to SWindPat.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Tools.HelpButton import HelpButton
from .....GUI.Tools.MPLCanvas import MPLCanvas2

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SWindPat(object):
    def setupUi(self, SWindPat):
        if not SWindPat.objectName():
            SWindPat.setObjectName(u"SWindPat")
        SWindPat.resize(650, 550)
        SWindPat.setMinimumSize(QSize(650, 550))
        self.verticalLayout_4 = QVBoxLayout(SWindPat)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.b_help = HelpButton(SWindPat)
        self.b_help.setObjectName(u"b_help")
        self.b_help.setPixmap(QPixmap(u":/images/images/icon/help_16.png"))

        self.horizontalLayout_2.addWidget(self.b_help)

        self.c_wind_type = QComboBox(SWindPat)
        self.c_wind_type.addItem("")
        self.c_wind_type.addItem("")
        self.c_wind_type.addItem("")
        self.c_wind_type.addItem("")
        self.c_wind_type.addItem("")
        self.c_wind_type.setObjectName(u"c_wind_type")

        self.horizontalLayout_2.addWidget(self.c_wind_type)

        self.in_qs = QLabel(SWindPat)
        self.in_qs.setObjectName(u"in_qs")

        self.horizontalLayout_2.addWidget(self.in_qs)

        self.si_qs = QSpinBox(SWindPat)
        self.si_qs.setObjectName(u"si_qs")

        self.horizontalLayout_2.addWidget(self.si_qs)

        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.verticalLayout_4.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.w_viewer = MPLCanvas2(SWindPat)
        self.w_viewer.setObjectName(u"w_viewer")

        self.horizontalLayout_5.addWidget(self.w_viewer)

        self.widget = QWidget(SWindPat)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_2 = QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_coil_pitch = QLabel(self.widget)
        self.in_coil_pitch.setObjectName(u"in_coil_pitch")

        self.horizontalLayout.addWidget(self.in_coil_pitch)

        self.si_coil_pitch = QSpinBox(self.widget)
        self.si_coil_pitch.setObjectName(u"si_coil_pitch")

        self.horizontalLayout.addWidget(self.si_coil_pitch)

        self.verticalLayout.addLayout(self.horizontalLayout)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.in_Nslot = QLabel(self.widget)
        self.in_Nslot.setObjectName(u"in_Nslot")
        self.in_Nslot.setMinimumSize(QSize(0, 0))
        self.in_Nslot.setMaximumSize(QSize(16777215, 16777215))

        self.horizontalLayout_4.addWidget(self.in_Nslot)

        self.si_Nslot = QSpinBox(self.widget)
        self.si_Nslot.setObjectName(u"si_Nslot")
        self.si_Nslot.setMinimumSize(QSize(60, 0))

        self.horizontalLayout_4.addWidget(self.si_Nslot)

        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.is_reverse = QCheckBox(self.widget)
        self.is_reverse.setObjectName(u"is_reverse")

        self.verticalLayout.addWidget(self.is_reverse)

        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.groupBox = QGroupBox(self.widget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setMinimumSize(QSize(200, 200))
        self.verticalLayout_3 = QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.out_shape = QLabel(self.groupBox)
        self.out_shape.setObjectName(u"out_shape")
        self.out_shape.setMinimumSize(QSize(175, 0))

        self.verticalLayout_3.addWidget(self.out_shape)

        self.out_ms = QLabel(self.groupBox)
        self.out_ms.setObjectName(u"out_ms")

        self.verticalLayout_3.addWidget(self.out_ms)

        self.out_Nperw = QLabel(self.groupBox)
        self.out_Nperw.setObjectName(u"out_Nperw")

        self.verticalLayout_3.addWidget(self.out_Nperw)

        self.verticalLayout_2.addWidget(self.groupBox)

        self.horizontalLayout_5.addWidget(self.widget)

        self.verticalLayout_4.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalSpacer_3 = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_3.addItem(self.horizontalSpacer_3)

        self.b_preview = QPushButton(SWindPat)
        self.b_preview.setObjectName(u"b_preview")

        self.horizontalLayout_3.addWidget(self.b_preview)

        self.b_previous = QPushButton(SWindPat)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout_3.addWidget(self.b_previous)

        self.b_next = QPushButton(SWindPat)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout_3.addWidget(self.b_next)

        self.verticalLayout_4.addLayout(self.horizontalLayout_3)

        self.retranslateUi(SWindPat)

        QMetaObject.connectSlotsByName(SWindPat)

    # setupUi

    def retranslateUi(self, SWindPat):
        SWindPat.setWindowTitle(QCoreApplication.translate("SWindPat", u"Form", None))
        self.b_help.setText("")
        self.c_wind_type.setItemText(
            0,
            QCoreApplication.translate(
                "SWindPat", u"Double Layer Concentrated Orthoradial", None
            ),
        )
        self.c_wind_type.setItemText(
            1,
            QCoreApplication.translate("SWindPat", u"Single Layer Concentrated", None),
        )
        self.c_wind_type.setItemText(
            2, QCoreApplication.translate("SWindPat", u"Double Layer Distributed", None)
        )
        self.c_wind_type.setItemText(
            3, QCoreApplication.translate("SWindPat", u"Single Layer Distributed", None)
        )
        self.c_wind_type.setItemText(
            4,
            QCoreApplication.translate(
                "SWindPat", u"Double Layer Concentrated Radial", None
            ),
        )

        self.in_qs.setText(QCoreApplication.translate("SWindPat", u"qs :", None))
        self.in_coil_pitch.setText(
            QCoreApplication.translate("SWindPat", u"coil_pitch", None)
        )
        self.in_Nslot.setText(
            QCoreApplication.translate("SWindPat", u"Nslot_shift", None)
        )
        self.is_reverse.setText(
            QCoreApplication.translate("SWindPat", u"reverse", None)
        )
        self.groupBox.setTitle(QCoreApplication.translate("SWindPat", u"Output", None))
        self.out_shape.setText(
            QCoreApplication.translate("SWindPat", u"Winding Matrix Shape : ", None)
        )
        self.out_ms.setText(
            QCoreApplication.translate("SWindPat", u"ms = Zs / (2*p*qs) = ?", None)
        )
        # if QT_CONFIG(tooltip)
        self.out_Nperw.setToolTip(
            QCoreApplication.translate("SWindPat", u"Winding periodicity", None)
        )
        # endif // QT_CONFIG(tooltip)
        self.out_Nperw.setText(QCoreApplication.translate("SWindPat", u"Nperw :", None))
        self.b_preview.setText(QCoreApplication.translate("SWindPat", u"Preview", None))
        self.b_previous.setText(
            QCoreApplication.translate("SWindPat", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SWindPat", u"Next", None))

    # retranslateUi



# -*- coding: utf-8 -*-

from numpy import real, min as np_min, max as np_max
from numpy.linalg import norm

from ....Classes.MeshMat import MeshMat
from ....definitions import config_dict

COLOR_MAP = config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"]


def plot_contour(
    self,
    label=None,
    index=None,
    indices=None,
    is_surf=False,
    is_radial=False,
    is_center=False,
    clim=None,
    field_name=None,
    group_names=None,
    save_path=None,
    itime=0,
    is_show_fig=True,
):
    """Plot the contour of a field on a mesh using pyvista plotter.

    Parameters
    ----------
    self : MeshSolution
        a MeshSolution object
    label : str
        a label
    index : int
        an index
    indices : list
        list of the points to extract (optional)
    is_surf : bool
        field over outer surface
    is_radial : bool
        radial component only
    is_center : bool
        field at cell-centers
    clim : list
        a list of 2 elements for the limits of the colorbar
    field_name : str
        title of the field to display on plot
    group_names : list
        a list of str corresponding to group name(s)
    save_path : str
        path to save the figure
    itime : int
        index of the time step to be plotted
    is_show_fig : bool
        To call show at the end of the method
    Returns
    -------
    """
    if group_names is not None:
        meshsol_grp = self.get_group(group_names)
        meshsol_grp.plot_contour(
            label=label,
            index=index,
            indices=indices,
            is_surf=is_surf,
            is_radial=is_radial,
            is_center=is_center,
            clim=clim,
            field_name=field_name,
            group_names=None,
            save_path=save_path,
            itime=itime,
        )
    else:
        if save_path is None:
            try:
                import pyvistaqt as pv

                is_pyvistaqt = True
            except:
                import pyvista as pv

                is_pyvistaqt = False
        else:
            import pyvista as pv

            is_pyvistaqt = False

        # Get the mesh
        mesh = self.get_mesh(label=label, index=index)
        mesh_pv = mesh.get_mesh_pv()

        # Get the field
        field = self.get_field(
            label=label,
            index=index,
            indices=indices,
            is_surf=is_surf,
            is_radial=is_radial,
            is_center=is_center,
        )

        # Extract time index
        if len(field.shape) > 2:
            # Extract time index
            if field.shape[1] > 3:
                field = field[itime, ...]
            # Compute norm
            if field.shape[-1] == 2 or field.shape[-1] == 3:
                field = norm(field, axis=-1)

        elif len(field.shape) > 1:
            # Extract time index
            if field.shape[1] > 3:
                field = field[itime, :]

        if field_name is None:
            if label is not None:
                field_name = label
            elif self.get_solution(index=index).label is not None:
                field_name = self.get_solution(index=index).label
            else:
                field_name = "Field"

        # Add field to mesh
        if is_surf:
            surf = mesh_pv.get_surf(indices=indices)
            surf[field_name] = real(field)
            mesh_field = surf
        else:
            mesh_pv[field_name] = real(field)
            mesh_field = mesh_pv

        # Configure plot
        if is_pyvistaqt:
            p = pv.BackgroundPlotter()
            p.set_background("white")
        else:
            pv.set_plot_theme("document")
            p = pv.Plotter(notebook=False)
        sargs = dict(
            interactive=True,
            title_font_size=20,
            label_font_size=16,
            font_family="arial",
            color="black",
        )
        p.add_mesh(
            mesh_field,
            scalars=field_name,
            show_edges=False,
            cmap=COLOR_MAP,
            clim=clim,
            scalar_bar_args=sargs,
        )
        if self.dimension == 2:
            p.view_xy()
        if save_path is None and is_show_fig:
            p.show()
        elif save_path is not None and is_show_fig:
            p.show(interactive=False, screenshot=save_path)



from ....Methods.Slot.Slot import SlotCheckError


class S10_W01CheckError(SlotCheckError):
    """ """

    pass


class S10_W02CheckError(SlotCheckError):
    """ """

    pass


class S10_H1rCheckError(SlotCheckError):
    """ """

    pass



# -*- coding: utf-8 -*-
from numpy import arange
from meshio import read
from os.path import join, split, splitext

from SciDataTool import DataTime, Data1D, VectorField

from ....Classes.SolutionData import SolutionData
from ....Classes.SolutionVector import SolutionVector
from ....Classes.MeshSolution import MeshSolution
from ....Classes.MeshVTK import MeshVTK

from ....Methods.Elmer.ElmerResultsVTU import ElmerResultsVTUError


# TODO add groups, see get_meshsolution of MagFEMM


def build_meshsolution(self):
    """Get the mesh and solution data from an Elmer VTU results file

    Parameters
    ----------
    self : ElmerResultsVTU
        a ElmerResultsVTU object

    Returns
    -------
    success: bool
        Information if meshsolution could be created

    """
    # create meshsolution
    meshsol = MeshSolution(label=self.label)

    # get the mesh
    save_path, fn = split(self.file_path)
    file_name, file_ext = splitext(fn)
    if file_ext != ".vtu":
        raise ElmerResultsVTUError("ElmerResultsVTU: Results file must be of type VTU.")

    meshvtk = MeshVTK(path=save_path, name=file_name, format="vtu")
    # TODO maybe convert to MeshMat before
    meshsol.mesh = [meshvtk]

    # get the solution data on the mesh
    meshsolvtu = read(self.file_path)
    pt_data = meshsolvtu.point_data  # point_data is of type dict

    # setup axes
    indices = arange(meshsolvtu.points.shape[0])
    Indices = Data1D(name="indice", values=indices, is_components=True)

    # store only data from store dict if available
    comp_ext = ["x", "y", "z"]

    sol_list = []  # list of solutions

    for key, value in pt_data.items():
        # check if value should be stored
        if key in self.store_dict.keys():
            siz = value.shape[1]
            # only regard max. 3 components
            if siz > 3:
                logger.warning(
                    f'ElmerResultsVTU.build_meshsolution(): size of data "{key}" > 3'
                    + " - "
                    + "Data will be truncated."
                )
                siz = 3

            components = []
            comp_name = []

            # loop though components
            for i in range(siz):
                # setup name, symbol and component name extension
                if siz == 1:
                    ext = ""
                else:
                    ext = comp_ext[i]

                # setup data object
                data = DataTime(
                    name=self.store_dict[key]["name"] + " " + ext,
                    unit=self.store_dict[key]["unit"],
                    symbol=self.store_dict[key]["symbol"] + ext,
                    axes=[Indices],
                    values=value[:, i],
                    normalizations={"ref": self.store_dict[key]["norm"]},
                )
                components.append(data)
                comp_name.append("comp_" + ext)

            # setup solution depending on number of field components
            if siz == 1:
                field = components[0]
                sol_list.append(
                    SolutionData(
                        field=field,
                        type_cell="point",
                        label=self.store_dict[key]["symbol"],
                    )
                )
            else:
                comps = {}
                for i in range(siz):
                    comps[comp_name[i]] = components[i]
                field = VectorField(
                    name=self.store_dict[key]["name"],
                    symbol=self.store_dict[key]["symbol"],
                    components=comps,
                )
                sol_list.append(
                    SolutionVector(
                        field=field,
                        type_cell="point",
                        label=self.store_dict[key]["symbol"],
                    )
                )

    meshsol.solution = sol_list

    return meshsol
